@@ -8,11 +8,11 @@
 class ATermStub : public ATermBeam
 {
 public:
-	ATermStub(casacore::MeasurementSet&, size_t /*width*/, size_t /*height*/, double /*dl*/, double /*dm*/, double /*phaseCentreDL*/, double /*phaseCentreDM*/, const std::string& /*dataColumnName*/)
+	ATermStub(casacore::MeasurementSet&, const CoordinateSystem&, const std::string& /*dataColumnName*/)
 	{
 		throw std::runtime_error("ATerm not implemented -- did you forget to turn specific beam options on during the compilation?");
 	}
-	virtual bool calculateBeam(std::complex<float>* /*buffer*/, double /*time*/, double /*frequency*/)
+	virtual bool calculateBeam(std::complex<float>* /*buffer*/, double /*time*/, double /*frequency*/, size_t /*fieldId*/)
 	{
 		throw std::runtime_error("ATerm not implemented -- did you forget to turn specific beam options on during the compilation?");
 	}
