@@ -1,13 +1,13 @@
 #include "lofarbeamterm.h"
 
-#include "../banddata.h"
-#include "../lane.h"
-#include "../matrix2x2.h"
+#include <aocommon/banddata.h>
+#include <aocommon/matrix2x2.h>
+
 #include "../system.h"
 
 #include "../lofar/lofarbeamkeywords.h"
 
-#include "../units/imagecoordinates.h"
+#include <aocommon/imagecoordinates.h>
 
 #include "../wsclean/logger.h"
 
@@ -17,16 +17,19 @@
 #include <casacore/measures/Measures/MEpoch.h>
 #include <casacore/tables/Tables/TableRecord.h>
 
+#include <aocommon/lane.h>
+
 #include <algorithm>
 
 using namespace LOFAR::StationResponse;
+using namespace aocommon;
 
-LofarBeamTerm::LofarBeamTerm(casacore::MeasurementSet& ms, size_t width, size_t height, double dl, double dm, double phaseCentreDL, double phaseCentreDM, const std::string& dataColumnName) :
-	_width(width),
-	_height(height),
-	_dl(dl), _dm(dm),
-	_phaseCentreDL(phaseCentreDL),
-	_phaseCentreDM(phaseCentreDM),
+LofarBeamTerm::LofarBeamTerm(casacore::MeasurementSet& ms, const CoordinateSystem& coordinateSystem, const std::string& dataColumnName) :
+	_width(coordinateSystem.width),
+	_height(coordinateSystem.height),
+	_dl(coordinateSystem.dl), _dm(coordinateSystem.dm),
+	_phaseCentreDL(coordinateSystem.phaseCentreDL),
+	_phaseCentreDM(coordinateSystem.phaseCentreDM),
 	_useDifferentialBeam(false),
 	_useChannelFrequency(true)
 {
@@ -37,7 +40,7 @@ LofarBeamTerm::LofarBeamTerm(casacore::MeasurementSet& ms, size_t width, size_t
 	_nThreads = std::min(nCPUs, nStations);
 	_threads.resize(_nThreads);
 
-	casacore::MPosition::ROScalarColumn antPosColumn(aTable, aTable.columnName(casacore::MSAntennaEnums::POSITION));
+	casacore::MPosition::ScalarColumn antPosColumn(aTable, aTable.columnName(casacore::MSAntennaEnums::POSITION));
 	_arrayPos = antPosColumn(0);
 	_stations.resize(aTable.nrow());
 	readStations(ms, _stations.begin());
@@ -46,9 +49,11 @@ LofarBeamTerm::LofarBeamTerm(casacore::MeasurementSet& ms, size_t width, size_t
 	_subbandFrequency = band.CentreFrequency();
 	
 	casacore::MSField fieldTable(ms.field());
+	if(fieldTable.nrow() != 1)
+		throw std::runtime_error("Set has multiple fields");
 	
-	casacore::MEpoch::ROScalarColumn timeColumn(ms, ms.columnName(casacore::MSMainEnums::TIME));
-	casacore::MDirection::ROScalarColumn phaseDirColumn(fieldTable, fieldTable.columnName(casacore::MSFieldEnums::PHASE_DIR));
+	casacore::MEpoch::ScalarColumn timeColumn(ms, ms.columnName(casacore::MSMainEnums::TIME));
+	casacore::MDirection::ScalarColumn phaseDirColumn(fieldTable, fieldTable.columnName(casacore::MSFieldEnums::PHASE_DIR));
 	casacore::MDirection phaseDir = phaseDirColumn(0);
 	casacore::MEpoch curtime = timeColumn(0);
 	casacore::MeasFrame frame(_arrayPos, curtime);
@@ -59,8 +64,6 @@ LofarBeamTerm::LofarBeamTerm(casacore::MeasurementSet& ms, size_t width, size_t
 	_phaseCentreDec = j2000Val[1];
 
 	casacore::ScalarMeasColumn<casacore::MDirection> delayDirColumn(fieldTable, casacore::MSField::columnName(casacore::MSFieldEnums::DELAY_DIR));
-	if(fieldTable.nrow() != 1)
-		throw std::runtime_error("Set has multiple fields");
 	_delayDir = delayDirColumn(0);
 	
 	if(fieldTable.tableDesc().isColumn("LOFAR_TILE_BEAM_DIR")) {
@@ -71,6 +74,8 @@ LofarBeamTerm::LofarBeamTerm(casacore::MeasurementSet& ms, size_t width, size_t
 	}
 	
 	_useDifferentialBeam = LOFARBeamKeywords::GetPreappliedBeamDirection(ms, dataColumnName, _useDifferentialBeam, _preappliedBeamDir);
+	Logger::Debug << "Tile direction: " << RaDecCoord::RaDecToString(_tileBeamDir.getAngle().getValue()[0], _tileBeamDir.getAngle().getValue()[1]) << '\n';
+	Logger::Debug << "Delay direction: " << RaDecCoord::RaDecToString(_delayDir.getAngle().getValue()[0], _delayDir.getAngle().getValue()[1]) << '\n';
 }
 
 void setITRFVector(const casacore::MDirection& itrfDir, LOFAR::StationResponse::vector3r_t& itrf)
@@ -81,17 +86,11 @@ void setITRFVector(const casacore::MDirection& itrfDir, LOFAR::StationResponse::
 	itrf[2] = itrfVal[2];
 }
 
-bool LofarBeamTerm::calculateBeam(std::complex<float>* buffer, double time, double frequency)
+bool LofarBeamTerm::calculateBeam(std::complex<float>* buffer, double time, double frequency, size_t)
 {
-	ao::lane<size_t> lane(_nThreads);
+	aocommon::Lane<size_t> lane(_nThreads);
 	_lane = &lane;
 
-	/*casacore::MDirection::Ref itrfRef(casacore::MDirection::ITRF, frame);
-	casacore::MDirection::Convert j2000ToITRFRef(j2000Ref, itrfRef);
-	casacore::MEpoch timeEpoch(casacore::Quantity(time, "s"));
-	casacore::MeasFrame frame(_arrayPos, timeEpoch);
-	casacore::MDirection::Ref j2000Ref(casacore::MDirection::J2000, frame);
-*/
 	LOFAR::StationResponse::ITRFConverter itrfConverter(time);
 	setITRFVector(itrfConverter.toDirection(_delayDir), _station0);
 	setITRFVector(itrfConverter.toDirection(_tileBeamDir), _tile0);
@@ -171,14 +170,12 @@ void LofarBeamTerm::calcThread(std::complex<float>* buffer, double time, double
 		size_t y = job_id / _stations.size();
 		for(size_t x=0; x!=_width; ++x)
 		{
-			double l, m, n, ra, dec;
+			double l, m, n;
 			ImageCoordinates::XYToLM(x, y, _dl, _dm, _width, _height, l, m);
 			l += _phaseCentreDL;
 			m += _phaseCentreDM;
 			n = sqrt(1.0 - l*l - m*m);
 
-			ImageCoordinates::LMToRaDec(l, m, _phaseCentreRA, _phaseCentreDec, ra, dec);
-
 			vector3r_t itrfDirection;
 
 			itrfDirection[0] = l*_l_vector_itrf[0] + m*_m_vector_itrf[0] + n*_n_vector_itrf[0];
