@@ -4,9 +4,8 @@
 #include "imageset.h"
 
 #include "../image.h"
-#include "../uvector.h"
 
-#include "../wsclean/imagebufferallocator.h"
+#include <aocommon/uvector.h>
 
 #include <vector>
 
@@ -21,8 +20,7 @@ public:
 		_nFrequencies(imageSet.size()),
 		_componentsAddedSinceLastMerge(0),
 		_maxComponentsBeforeMerge(100000),
-		_listPerScale(1),
-		_allocator(imageSet.Allocator())
+		_listPerScale(1)
 	{
 		loadFromImageSet(imageSet, 0);
 	}
@@ -30,13 +28,12 @@ public:
 	/**
 	 * Constructor for multi-scale clean
 	 */
-	ComponentList(size_t width, size_t height, size_t nScales, size_t nFrequencies, ImageBufferAllocator& allocator) :
+	ComponentList(size_t width, size_t height, size_t nScales, size_t nFrequencies) :
 		_width(width), _height(height),
 		_nFrequencies(nFrequencies),
 		_componentsAddedSinceLastMerge(0),
 		_maxComponentsBeforeMerge(100000),
-		_listPerScale(nScales),
-		_allocator(allocator)
+		_listPerScale(nScales)
 	{
 	}
 	
@@ -124,23 +121,23 @@ private:
 	 * component, such that _positions[i] corresponds with the values
 	 * starting at _values[i * _nFrequencies].
 	 */
-		ao::uvector<double> values;
-		ao::uvector<Position> positions;
+		aocommon::UVector<double> values;
+		aocommon::UVector<Position> positions;
 	};
 	
-	void write(const std::string& filename, const class DeconvolutionAlgorithm& algorithm, const ao::uvector<double>& scaleSizes, long double pixelScaleX, long double pixelScaleY, long double phaseCentreRA, long double phaseCentreDec);
+	void write(const std::string& filename, const class DeconvolutionAlgorithm& algorithm, const aocommon::UVector<double>& scaleSizes, long double pixelScaleX, long double pixelScaleY, long double phaseCentreRA, long double phaseCentreDec);
 	
 	void loadFromImageSet(ImageSet& imageSet, size_t scaleIndex);
   
 	void mergeDuplicates(size_t scaleIndex)
 	{
 		ScaleList& list = _listPerScale[scaleIndex];
-		ao::uvector<double> newValues;
-		ao::uvector<Position> newPositions;
+		aocommon::UVector<double> newValues;
+		aocommon::UVector<Position> newPositions;
 		
 		std::vector<Image> images(_nFrequencies);
 		for(Image& image : images)
-			image = Image(_width, _height, 0.0, _allocator);
+			image = Image(_width, _height, 0.0);
 		size_t valueIndex = 0;
 		for(size_t index=0; index!=list.positions.size(); ++index)
 		{
@@ -184,7 +181,6 @@ private:
 	size_t _componentsAddedSinceLastMerge;
 	size_t _maxComponentsBeforeMerge;
 	std::vector<ScaleList> _listPerScale;
-	ImageBufferAllocator& _allocator;
 };
 
 #endif
