@@ -2,23 +2,24 @@
 
 #include "subminorloop.h"
 
-#include "../lane.h"
+#include <aocommon/lane.h>
 
 #include "../multiscale/threadeddeconvolutiontools.h"
 
 #include "../units/fluxdensity.h"
 
+#include "../wsclean/logger.h"
+
 #include <boost/thread/thread.hpp>
 
-GenericClean::GenericClean(ImageBufferAllocator& allocator, class FFTWManager& fftwManager, bool useSubMinorOptimization) :
+GenericClean::GenericClean(class FFTWManager& fftwManager, bool useSubMinorOptimization) :
 	_convolutionPadding(1.1),
 	_useSubMinorOptimization(useSubMinorOptimization),
-	_allocator(allocator),
 	_fftwManager(fftwManager)
 {
 }
 
-double GenericClean::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet& modelSet, const ao::uvector<const double*>& psfs, size_t width, size_t height, bool& reachedMajorThreshold)
+double GenericClean::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet& modelSet, const aocommon::UVector<const double*>& psfs, size_t width, size_t height, bool& reachedMajorThreshold)
 {
 	const size_t iterationCounterAtStart = _iterationNumber;
 	if(_stopOnNegativeComponent)
@@ -32,20 +33,20 @@ double GenericClean::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet& modelSe
 	if(_convolutionHeight%2 != 0)
 		++_convolutionHeight;
 	
-	ImageBufferAllocator::Ptr integrated, scratchA, scratchB;
-	_allocator.Allocate(width*height, integrated);
-	_allocator.Allocate(_convolutionWidth*_convolutionHeight, scratchA);
-	_allocator.Allocate(_convolutionWidth*_convolutionHeight, scratchB);
+	Image
+		integrated(width, height),
+		scratchA(_convolutionWidth, _convolutionHeight),
+		scratchB(_convolutionWidth, _convolutionHeight);
 	dirtySet.GetLinearIntegrated(integrated.data());
 	size_t componentX=0, componentY=0;
 	boost::optional<double> maxValue = findPeak(integrated.data(), scratchA.data(), componentX, componentY);
 	if(!maxValue)
 	{
-		Logger::Info << "No peak found.\n";
+		_logReceiver->Info << "No peak found.\n";
 		reachedMajorThreshold = false;
 		return 0.0;
 	}
-	Logger::Info << "Initial peak: " << peakDescription(integrated.data(), componentX, componentY) << '\n';
+	_logReceiver->Info << "Initial peak: " << peakDescription(integrated.data(), componentX, componentY) << '\n';
 	double firstThreshold = this->_threshold;
 	double majorIterThreshold = std::max(
 		MajorIterThreshold(),
@@ -53,16 +54,16 @@ double GenericClean::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet& modelSe
 	if(majorIterThreshold > firstThreshold)
 	{
 		firstThreshold = majorIterThreshold;
-		Logger::Info << "Next major iteration at: " << FluxDensity::ToNiceString(majorIterThreshold) << '\n';
+		_logReceiver->Info << "Next major iteration at: " << FluxDensity::ToNiceString(majorIterThreshold) << '\n';
 	}
 	else if(this->_mGain != 1.0) {
-		Logger::Info << "Major iteration threshold reached global threshold of " << FluxDensity::ToNiceString(this->_threshold) << ": final major iteration.\n";
+		_logReceiver->Info << "Major iteration threshold reached global threshold of " << FluxDensity::ToNiceString(this->_threshold) << ": final major iteration.\n";
 	}
 	
 	if(_useSubMinorOptimization)
 	{
 		size_t startIteration = _iterationNumber;
-		SubMinorLoop subMinorLoop(_width, _height, _convolutionWidth, _convolutionHeight);
+		SubMinorLoop subMinorLoop(_width, _height, _convolutionWidth, _convolutionHeight, *_logReceiver);
 		subMinorLoop.SetIterationInfo(_iterationNumber, MaxNIter());
 		subMinorLoop.SetThreshold(firstThreshold, firstThreshold*0.99);
 		subMinorLoop.SetGain(Gain());
@@ -82,7 +83,7 @@ double GenericClean::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet& modelSe
 		
 		_iterationNumber = subMinorLoop.CurrentIteration();
 		
-		Logger::Info << "Performed " << _iterationNumber << " iterations in total, " << (_iterationNumber - startIteration) << " in this major iteration with sub-minor optimization.\n";
+		_logReceiver->Info << "Performed " << _iterationNumber << " iterations in total, " << (_iterationNumber - startIteration) << " in this major iteration with sub-minor optimization.\n";
 		
 		for(size_t imageIndex=0; imageIndex!=dirtySet.size(); ++imageIndex)
 		{
@@ -100,7 +101,7 @@ double GenericClean::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet& modelSe
 		ThreadedDeconvolutionTools tools(_threadCount);
 		size_t peakIndex = componentX + componentY*_width;
 
-		ao::uvector<double> peakValues(dirtySet.size());
+		aocommon::UVector<double> peakValues(dirtySet.size());
 		
 		while(maxValue && fabs(*maxValue) > firstThreshold && this->_iterationNumber < this->_maxIter && !(maxValue<0.0 && this->_stopOnNegativeComponent))
 		{
@@ -108,7 +109,7 @@ double GenericClean::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet& modelSe
 				(this->_iterationNumber <= 100 && this->_iterationNumber % 10 == 0) ||
 				(this->_iterationNumber <= 1000 && this->_iterationNumber % 100 == 0) ||
 				this->_iterationNumber % 1000 == 0)
-				Logger::Info << "Iteration " << this->_iterationNumber << ": " << peakDescription(integrated.data(), componentX, componentY) << '\n';
+				_logReceiver->Info << "Iteration " << this->_iterationNumber << ": " << peakDescription(integrated.data(), componentX, componentY) << '\n';
 			
 			for(size_t i=0; i!=dirtySet.size(); ++i)
 				peakValues[i] = dirtySet[i][peakIndex];
@@ -135,7 +136,7 @@ double GenericClean::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet& modelSe
 	}
 	if(maxValue)
 	{
-		Logger::Info << "Stopped on peak " << FluxDensity::ToNiceString(*maxValue) << ", because ";
+		_logReceiver->Info << "Stopped on peak " << FluxDensity::ToNiceString(*maxValue) << ", because ";
 		bool
 			maxIterReached = _iterationNumber >= MaxNIter(),
 			finalThresholdReached = std::fabs(*maxValue) <= _threshold || maxValue == 0.0,
@@ -144,20 +145,20 @@ double GenericClean::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet& modelSe
 			didWork = (_iterationNumber-iterationCounterAtStart)!=0;
 		
 		if(maxIterReached)
-			Logger::Info << "maximum number of iterations was reached.\n";
+			_logReceiver->Info << "maximum number of iterations was reached.\n";
 		else if(finalThresholdReached)
-			Logger::Info << "the threshold was reached.\n";
+			_logReceiver->Info << "the threshold was reached.\n";
 		else if(negativeReached)
-			Logger::Info << "a negative component was found.\n";
+			_logReceiver->Info << "a negative component was found.\n";
 		else if(!didWork)
-			Logger::Info << "no iterations could be performed.\n";
+			_logReceiver->Info << "no iterations could be performed.\n";
 		else
-			Logger::Info << "the minor-loop threshold was reached. Continuing cleaning after inversion/prediction round.\n";
+			_logReceiver->Info << "the minor-loop threshold was reached. Continuing cleaning after inversion/prediction round.\n";
 		reachedMajorThreshold = mgainReached && didWork && !negativeReached && !finalThresholdReached;
 		return *maxValue;
 	}
 	else {
-		Logger::Info << "Deconvolution aborted.\n";
+		_logReceiver->Info << "Deconvolution aborted.\n";
 		reachedMajorThreshold = false;
 		return 0.0;
 	}
