@@ -5,21 +5,21 @@
 #include "imageset.h"
 #include "simpleclean.h"
 
-#include "../uvector.h"
+#include <aocommon/uvector.h>
 
 #include <boost/thread/thread.hpp>
 
 /**
  * This class implements a generalized version of Högbom clean. It performs a single-channel
- * or joined cleaning, depending on the number of images provided. It can use the Clark optimization
+ * or joined cleaning, depending on the number of images provided. It can use a Clark-like optimization
  * to speed up the cleaning. When multiple frequencies are provided, it can perform spectral fitting.
  */
 class GenericClean : public DeconvolutionAlgorithm
 {
 public:
-	explicit GenericClean(class ImageBufferAllocator& allocator, class FFTWManager& fftwManager, bool useSubMinorOptimization);
+	explicit GenericClean(class FFTWManager& fftwManager, bool useSubMinorOptimization);
 	
-	virtual double ExecuteMajorIteration(ImageSet& dirtySet, ImageSet& modelSet, const ao::uvector<const double*>& psfs, size_t width, size_t height, bool& reachedMajorThreshold) final override;
+	virtual double ExecuteMajorIteration(ImageSet& dirtySet, ImageSet& modelSet, const aocommon::UVector<const double*>& psfs, size_t width, size_t height, bool& reachedMajorThreshold) final override;
 	
 	virtual std::unique_ptr<DeconvolutionAlgorithm> Clone() const final override
 	{
@@ -40,7 +40,6 @@ private:
 		SimpleClean::PartialSubtractImage(image, _width, _height, psf, _width, _height, x, y, factor, startY, endY);
 	}
 	
-	class ImageBufferAllocator& _allocator;
 	class FFTWManager& _fftwManager;
 };
 
