@@ -1,11 +1,6 @@
 #ifndef IUWT_DECONVOLUTION_H
 #define IUWT_DECONVOLUTION_H
 
-#include <memory>
-#include <string>
-
-#include "../uvector.h"
-
 #include "deconvolutionalgorithm.h"
 #include "imageset.h"
 
@@ -13,6 +8,11 @@
 
 #include "../wsclean/imagingtable.h"
 
+#include <aocommon/uvector.h>
+
+#include <memory>
+#include <string>
+
 class IUWTDeconvolution : public DeconvolutionAlgorithm
 {
 public:
@@ -21,7 +21,7 @@ public:
 		_useSNRTest(false)
 	{ }
 	
-	virtual double ExecuteMajorIteration(ImageSet& dataImage, ImageSet& modelImage, const ao::uvector<const double*>& psfImages, size_t width, size_t height, bool& reachedMajorThreshold) final override
+	virtual double ExecuteMajorIteration(ImageSet& dataImage, ImageSet& modelImage, const aocommon::UVector<const double*>& psfImages, size_t width, size_t height, bool& reachedMajorThreshold) final override
 	{
 		IUWTDeconvolutionAlgorithm algorithm(_fftwManager, width, height, _gain, _mGain, _cleanBorderRatio, _allowNegativeComponents, _cleanMask, _threshold, _useSNRTest);
 		double val = algorithm.PerformMajorIteration(_iterationNumber, MaxNIter(), modelImage, dataImage, psfImages, reachedMajorThreshold);
