@@ -3,7 +3,12 @@
 
 #include "../fftwmanager.h"
 #include "../image.h"
-#include "../uvector.h"
+
+#include "../wsclean/primarybeamimageset.h"
+
+#include "controllablelog.h"
+
+#include <aocommon/uvector.h>
 
 #include <memory>
 #include <mutex>
@@ -32,7 +37,7 @@ public:
 	
 	void SetAlgorithm(std::unique_ptr<class DeconvolutionAlgorithm> algorithm);
 	
-	void SetRMSFactorImage(class Image&& image);
+	void SetRMSFactorImage(Image&& image);
 	
 	void SetThreshold(double threshold);
 	
@@ -45,7 +50,7 @@ public:
 	
 	void SetCleanMask(const bool* mask);
 	
-	void ExecuteMajorIteration(class ImageSet& dataImage, class ImageSet& modelImage, const ao::uvector<const double*>& psfImages, bool& reachedMajorThreshold);
+	void ExecuteMajorIteration(class ImageSet& dataImage, class ImageSet& modelImage, const aocommon::UVector<const double*>& psfImages, bool& reachedMajorThreshold);
 	
 	void FreeDeconvolutionAlgorithms()
 	{
@@ -60,31 +65,32 @@ public:
 	class FFTWManager& GetFFTWManager() { return _fftwManager; }
 	
 private:
-	void executeParallelRun(class ImageSet& dataImage, class ImageSet& modelImage, const ao::uvector<const double*>& psfImages, bool& reachedMajorThreshold);
+	void executeParallelRun(class ImageSet& dataImage, class ImageSet& modelImage, const aocommon::UVector<const double*>& psfImages, bool& reachedMajorThreshold);
 	
 	struct SubImage {
 		size_t index, x, y, width, height;
-		ao::uvector<bool> mask;
+		aocommon::UVector<bool> mask;
 		double peak;
 		bool reachedMajorThreshold;
 	};
 		
-	void runSubImage(SubImage& subImg, ImageSet& dataImage, class ImageSet& modelImage, const ao::uvector<const double*>& psfImages, double majorIterThreshold, bool findPeakOnly, std::mutex* mutex);
+	void runSubImage(SubImage& subImg, ImageSet& dataImage, class ImageSet& modelImage, const aocommon::UVector<const double*>& psfImages, double majorIterThreshold, bool findPeakOnly, std::mutex* mutex);
 	
 	void correctChannelForPB(class ComponentList& list, const class ImagingTableEntry& entry) const;
 	
-	void loadAveragePrimaryBeam(class PrimaryBeamImageSet& beamImages, size_t imageIndex, const class ImagingTable& table) const;
+	PrimaryBeamImageSet loadAveragePrimaryBeam(size_t imageIndex, const class ImagingTable& table) const;
 	
 	void writeSourceList(ComponentList& componentList, const std::string& filename, long double phaseCentreRA, long double phaseCentreDec) const;
 	
 	FFTWManager _fftwManager;
 	std::vector<std::unique_ptr<class DeconvolutionAlgorithm>> _algorithms;
+	FacetLogSet _logs;
 	size_t _horImages, _verImages;
 	const WSCleanSettings& _settings;
 	ImageBufferAllocator* _allocator;
 	const bool* _mask;
 	bool _trackPerScaleMasks, _usePerScaleMasks;
-	std::vector<ao::uvector<bool>> _scaleMasks;
+	std::vector<aocommon::UVector<bool>> _scaleMasks;
 	std::unique_ptr<class ComponentList> _componentList;
 	Image _rmsImage;
 };
