@@ -1,14 +1,13 @@
 #ifndef BEAM_IMAGE_MAKER
 #define BEAM_IMAGE_MAKER
 
-#include <set>
-
-#include "../polarization.h"
-#include "../uvector.h"
+#include "../hmatrix4x4.h"
+#include <aocommon/polarization.h>
 #include "../wsclean/imagingtable.h"
-#include "../wsclean/imagebufferallocator.h"
 #include "../wsclean/primarybeamimageset.h"
 
+#include <aocommon/uvector.h>
+
 #include <casacore/ms/MeasurementSets/MeasurementSet.h>
 #include <casacore/measures/Measures/MDirection.h>
 
@@ -16,11 +15,13 @@
 #include <StationResponse/Station.h>
 #endif
 
+#include <set>
+
 class LBeamImageMaker
 {
 public:
-	LBeamImageMaker(const ImagingTableEntry* tableEntry, ImageBufferAllocator* allocator) :
-	_tableEntry(tableEntry), _allocator(allocator),
+	LBeamImageMaker(const ImagingTableEntry* tableEntry) :
+	_tableEntry(tableEntry),
 	_undersample(8), _secondsBeforeBeamUpdate(1800),
 	_useDifferentialBeam(false)
 	{
@@ -48,7 +49,7 @@ public:
 		_phaseCentreDM = phaseCentreDM;
 	}
 	
-	void Make(PrimaryBeamImageSet& beamImages);
+	PrimaryBeamImageSet Make();
 	
 	void SetUseDifferentialBeam(bool useDifferentialBeam) {
 		_useDifferentialBeam = useDifferentialBeam;
@@ -59,6 +60,15 @@ public:
 		_undersample = undersamplingFactor;
 	}
 	
+	void SetSaveIntermediateImages(bool saveIntermediateImages) {
+		_saveIntermediateImages = saveIntermediateImages;
+	}
+	
+	void SetSecondsBeforeBeamUpdate(size_t seconds)
+	{
+		_secondsBeforeBeamUpdate = seconds;
+	}
+	
 private:
 #ifdef HAVE_LOFAR_BEAM
 	class WeightMatrix
@@ -82,16 +92,16 @@ private:
 		}
 	private:
 		size_t _nAntenna;
-		ao::uvector<double> _weights;
+		aocommon::UVector<double> _weights;
 	};
 	
-	void makeBeamForMS(PrimaryBeamImageSet& beamImages, MSProvider& msProvider, const MSSelection& selection, double centralFrequency);
+	void makeBeamForMS(std::vector<HMC4x4>& _matrices, MSProvider& msProvider, const MSSelection& selection, double centralFrequency);
 
-	void makeBeamSnapshot(const std::vector<LOFAR::StationResponse::Station::Ptr>& stations, const ao::uvector<double>& weights, double** imgPtr, double time, double frequency, double subbandFrequency, const casacore::MeasFrame& frame);
+	void makeBeamSnapshot(const std::vector<LOFAR::StationResponse::Station::Ptr>& stations, const WeightMatrix& weights, HMC4x4* matrices, double time, double frequency, double subbandFrequency, const casacore::MeasFrame& frame);
 	
-	void calculateStationWeights(const class ImageWeights& imageWeights, double& totalWeight, ao::uvector<double>& weights, WeightMatrix& baselineWeights, SynchronizedMS& ms, MSProvider& msProvider, const MSSelection& selection, double endTime);
+	void calculateStationWeights(const class ImageWeights& imageWeights, double& totalWeight, WeightMatrix& baselineWeights, SynchronizedMS& ms, MSProvider& msProvider, const MSSelection& selection, double endTime);
 	
-	void logWeights(casacore::MeasurementSet& ms, const ao::uvector<double>& weights);
+	void logWeights(casacore::MeasurementSet& ms, const aocommon::UVector<double>& weights);
 #endif
 	
 	struct MSProviderInfo
@@ -114,7 +124,7 @@ private:
 	size_t _undersample, _secondsBeforeBeamUpdate;
 	double _pixelSizeX, _pixelSizeY, _phaseCentreRA, _phaseCentreDec, _phaseCentreDL, _phaseCentreDM;
 	double _sPixelSizeX, _sPixelSizeY, _totalWeightSum;
- 	bool _useDifferentialBeam;
+ 	bool _useDifferentialBeam, _saveIntermediateImages;
 	casacore::MDirection _delayDir, _preappliedDir, _tileBeamDir;
 };
 
