@@ -3,12 +3,12 @@
 
 #include "msrowprovider.h"
 
-#include "../uvector.h"
+#include <aocommon/uvector.h>
 
 class AveragingMSRowProvider : public MSRowProvider
 {
 public:
-	AveragingMSRowProvider(double nWavelengthsAveraging, const string& msPath, const MSSelection& selection, const std::map<size_t,size_t>& selectedDataDescIds, const std::string& dataColumnName, bool requireModel);
+	AveragingMSRowProvider(double nWavelengthsAveraging, const string& msPath, const MSSelection& selection, const std::map<size_t,size_t>& selectedDataDescIds, size_t fieldId, const std::string& dataColumnName, bool requireModel);
 	
 	virtual bool AtEnd() const final override {
 		return _flushPosition >= _nElements;
@@ -16,7 +16,7 @@ public:
 	
 	virtual void NextRow() final override;
 	
-	virtual void ReadData(DataArray& data, FlagArray& flags, WeightArray& weights, double& u, double& v, double& w, uint32_t& dataDescId, uint32_t& antenna1, uint32_t& antenna2, double& time) final override;
+	virtual void ReadData(DataArray& data, FlagArray& flags, WeightArray& weights, double& u, double& v, double& w, uint32_t& dataDescId, uint32_t& antenna1, uint32_t& antenna2, uint32_t& fieldId, double& time) final override;
 	
 	virtual void ReadModel(DataArray& model) final override;
 	
@@ -102,9 +102,17 @@ private:
 				flags[i] = (weights[i]==0.0);
 				weights[i] = _weights[i];
 			}
-			for(size_t i=0; i!=3; ++i)
-				uvw[i] = _uvw[i] / _summedWeight;
-			time = _time / _summedWeight;
+			if(_summedWeight == 0.0)
+			{
+				for(size_t i=0; i!=3; ++i)
+					uvw[i] = 0.0;
+				time = 0.0;
+			}
+			else {
+				for(size_t i=0; i!=3; ++i)
+					uvw[i] = _uvw[i] / _summedWeight;
+				time = _time / _summedWeight;
+			}
 		}
 		
 		void Get(size_t n, std::complex<float>* data, std::complex<float>* modelData, bool* flags, float* weights, double* uvw, double& time)
@@ -116,9 +124,17 @@ private:
 				flags[i] = (weights[i]==0.0);
 				weights[i] = _weights[i];
 			}
-			for(size_t i=0; i!=3; ++i)
-				uvw[i] = _uvw[i] / _summedWeight;
-			time = _time / _summedWeight;
+			if(_summedWeight == 0.0)
+			{
+				for(size_t i=0; i!=3; ++i)
+					uvw[i] = 0.0;
+				time = 0.0;
+			}
+			else {
+				for(size_t i=0; i!=3; ++i)
+					uvw[i] = _uvw[i] / _summedWeight;
+				time = _time / _summedWeight;
+			}
 		}
 		
 		void Reset(size_t n)
@@ -148,9 +164,9 @@ private:
 	
 	typedef std::tuple<double, double, double> Pos;
 	
-	ao::uvector<size_t> _averagingFactors;
+	aocommon::UVector<size_t> _averagingFactors;
 	std::vector<AveragingBuffer> _buffers;
-	ao::uvector<size_t> _spwIndexToDataDescId;
+	aocommon::UVector<size_t> _spwIndexToDataDescId;
 	
 	size_t _nAntennae;
 	
@@ -168,6 +184,7 @@ private:
 	size_t _averagedRowCount;
 	size_t _averageFactorSum;
 	size_t _rowCount;
+	size_t _fieldId;
 };
 
 #endif
