@@ -16,7 +16,7 @@
 class ContiguousMS : public MSProvider
 {
 public:
-	ContiguousMS(const string& msPath, const std::string& dataColumnName, const MSSelection& selection, PolarizationEnum polOut, size_t dataDescIndex);
+	ContiguousMS(const string& msPath, const std::string& dataColumnName, const MSSelection& selection, aocommon::PolarizationEnum polOut, size_t dataDescIndex);
 	
 	ContiguousMS(const ContiguousMS&) = delete;
 	
@@ -59,19 +59,29 @@ public:
 	
 	void MakeIdToMSRowMapping(std::vector<size_t>& idToMSRow) final override;
 	
-	PolarizationEnum Polarization() final override { return _polOut; }
+	aocommon::PolarizationEnum Polarization() final override { return _polOut; }
+	
+	size_t NChannels() override;
+	
+	size_t NPolarizations() override;
+	
+	size_t NAntennas() override { return _nAntenna; }
+	
 private:
+	void open();
+	
 	size_t _row, _rowId;
 	size_t _timestep;
 	double _time;
 	int _dataDescId;
+	size_t _nAntenna;
 	bool _isMetaRead, _isDataRead, _isModelRead, _isWeightRead;
 	bool _isModelColumnPrepared;
 	size_t _startRow, _endRow;
 	std::vector<size_t> _idToMSRow;
-	std::vector<PolarizationEnum> _inputPolarizations;
+	std::vector<aocommon::PolarizationEnum> _inputPolarizations;
 	MSSelection _selection;
-	PolarizationEnum _polOut;
+	aocommon::PolarizationEnum _polOut;
 	std::string _msPath;
 	SynchronizedMS _ms;
 	MultiBandData _bandData;
@@ -83,8 +93,8 @@ private:
 	std::unique_ptr<casacore::ArrayColumn<float>> _weightSpectrumColumn;
 	std::unique_ptr<casacore::ArrayColumn<float>> _weightScalarColumn;
 	std::string _dataColumnName;
-	casacore::ROArrayColumn<casacore::Complex> _dataColumn;
-	casacore::ROArrayColumn<bool> _flagColumn;
+	casacore::ArrayColumn<casacore::Complex> _dataColumn;
+	casacore::ArrayColumn<bool> _flagColumn;
 	std::unique_ptr<casacore::ArrayColumn<casacore::Complex>> _modelColumn;
 	std::unique_ptr<casacore::ArrayColumn<float>> _imagingWeightsColumn;
 	
