@@ -1,21 +1,22 @@
 #ifndef PARTITIONED_MS
 #define PARTITIONED_MS
 
-#include <fstream>
-#include <string>
-#include <map>
+#include "msprovider.h"
+
+#include <aocommon/polarization.h>
+#include "../msselection.h"
+
+#include <aocommon/uvector.h>
 
 #include <casacore/ms/MeasurementSets/MeasurementSet.h>
 #include <casacore/tables/Tables/ArrayColumn.h>
 #include <casacore/tables/Tables/ScalarColumn.h>
 
-#include "../polarization.h"
-#include "../uvector.h"
-#include "../msselection.h"
+#include <fstream>
+#include <string>
+#include <map>
 
-#include "msprovider.h"
-
-class PartitionedMS : public MSProvider
+class PartitionedMS final : public MSProvider
 {
 public:
 	class Handle;
@@ -34,62 +35,71 @@ public:
 		}
 	};
 	
-	PartitionedMS(const Handle& handle, size_t partIndex, PolarizationEnum polarization, size_t bandIndex);
+	PartitionedMS(const Handle& handle, size_t partIndex, aocommon::PolarizationEnum polarization, size_t bandIndex);
 	
 	virtual ~PartitionedMS();
 	
 	PartitionedMS(const PartitionedMS&) = delete;
 	PartitionedMS& operator=(const PartitionedMS&) = delete;
 	
-	SynchronizedMS MS() final override
+	SynchronizedMS MS() override
 	{
-		return SynchronizedMS(_msPath.data());
+		return SynchronizedMS(_handle._data->_msPath.data());
 	}
 	
-	const std::string& DataColumnName() final override { return _handle._data->_dataColumnName; }
+	const std::string& DataColumnName() override { return _handle._data->_dataColumnName; }
 	
-	size_t RowId() const final override { return _currentRow; }
+	size_t RowId() const override { return _currentRow; }
 	
-	bool CurrentRowAvailable() final override;
+	bool CurrentRowAvailable() override;
 	
-	void NextRow() final override;
+	void NextRow() override;
 	
-	void Reset() final override;
+	void Reset() override;
 	
-	void ReadMeta(double& u, double& v, double& w, size_t& dataDescId) final override;
+	void ReadMeta(double& u, double& v, double& w, size_t& dataDescId) override;
 	
-	void ReadMeta(MetaData& metaData) final override;
+	void ReadMeta(MetaData& metaData) override;
 	
-	void ReadData(std::complex<float>* buffer) final override;
+	void ReadData(std::complex<float>* buffer) override;
 	
-	void ReadModel(std::complex<float>* buffer) final override;
+	void ReadModel(std::complex<float>* buffer) override;
 	
-	void WriteModel(size_t rowId, std::complex<float>* buffer) final override;
+	void WriteModel(size_t rowId, std::complex<float>* buffer) override;
 	
-	void WriteImagingWeights(size_t rowId, const float* buffer) final override;
+	void WriteImagingWeights(size_t rowId, const float* buffer) override;
 	
-	void ReadWeights(float* buffer) final override;
+	void ReadWeights(float* buffer) override;
 	
-	void ReadWeights(std::complex<float>* buffer) final override;
+	void ReadWeights(std::complex<float>* buffer) override;
 	
-	void ReopenRW() final override{ }
+	void ReopenRW() override { }
 	
-	double StartTime() final override { return _metaHeader.startTime; }
+	double StartTime() override { return _metaHeader.startTime; }
 	
-	void MakeIdToMSRowMapping(std::vector<size_t>& idToMSRow) final override;
-	
-	PolarizationEnum Polarization() final override { return _polarization; }
+	void MakeIdToMSRowMapping(std::vector<size_t>& idToMSRow) override;
 	
+	aocommon::PolarizationEnum Polarization() override { return _polarization; }
+
+	size_t NChannels() override { return _partHeader.channelCount; }
+	size_t NPolarizations() override { return _polarizationCountInFile; }
+	size_t NAntennas() override { return _handle._data->_nAntennas; }
+
 	static Handle Partition(const string& msPath, const std::vector<ChannelRange>& channels, class MSSelection& selection, const string& dataColumnName, bool includeModel, bool initialModelRequired, const class WSCleanSettings& settings);
 	
 	class Handle {
 	public:
 		Handle() = default;
 		
+		void Serialize(class SerialOStream& stream) const;
+		void Unserialize(class SerialIStream& stream);
+		
 		friend class PartitionedMS;
 	private:
 		struct HandleData
 		{
+			HandleData() : _isCopy(false) { }
+			
 			HandleData(
 				const std::string& msPath,
 				const string& dataColumnName,
@@ -97,18 +107,24 @@ public:
 				const std::vector<ChannelRange>& channels,
 				bool initialModelRequired,
 				bool modelUpdateRequired,
-				const std::set<PolarizationEnum>& polarizations,
-				const MSSelection& selection) :
+				const std::set<aocommon::PolarizationEnum>& polarizations,
+				const MSSelection& selection,
+				size_t nAntennas) :
 			_msPath(msPath), _dataColumnName(dataColumnName), _temporaryDirectory(temporaryDirectory), _channels(channels), _initialModelRequired(initialModelRequired), _modelUpdateRequired(modelUpdateRequired),
-			_polarizations(polarizations), _selection(selection) { }
+			_polarizations(polarizations), _selection(selection), _nAntennas(nAntennas), _isCopy(false) { }
 			
 			~HandleData();
 			
 			std::string _msPath, _dataColumnName, _temporaryDirectory;
 			std::vector<ChannelRange> _channels;
 			bool _initialModelRequired, _modelUpdateRequired;
-			std::set<PolarizationEnum> _polarizations;
+			std::set<aocommon::PolarizationEnum> _polarizations;
 			MSSelection _selection;
+			size_t _nAntennas;
+			bool _isCopy;
+			
+			void Serialize(class SerialOStream& stream) const;
+			void Unserialize(class SerialIStream& stream);
 		};
 		std::shared_ptr<HandleData> _data;
 		
@@ -119,9 +135,10 @@ public:
 			const std::vector<ChannelRange>& channels,
 			bool initialModelRequired,
 			bool modelUpdateRequired,
-			const std::set<PolarizationEnum>& polarizations,
-			const MSSelection& selection) :
-		_data(new HandleData(msPath, dataColumnName, temporaryDirectory, channels, initialModelRequired, modelUpdateRequired, polarizations, selection))
+			const std::set<aocommon::PolarizationEnum>& polarizations,
+			const MSSelection& selection,
+			size_t nAntennas) :
+		_data(new HandleData(msPath, dataColumnName, temporaryDirectory, channels, initialModelRequired, modelUpdateRequired, polarizations, selection, nAntennas))
 		{ }
 	};
 private:
@@ -130,18 +147,17 @@ private:
 	static void getDataDescIdMap(std::map<size_t,size_t>& dataDescIds, const std::vector<PartitionedMS::ChannelRange>& channels);
 	
 	Handle _handle;
-	std::string _msPath;
 	size_t _partIndex;
 	std::ifstream _metaFile, _weightFile, _dataFile;
 	char *_modelFileMap;
 	size_t _currentRow;
 	bool _readPtrIsOk, _metaPtrIsOk, _weightPtrIsOk;
-	ao::uvector<float> _weightBuffer, _imagingWeightBuffer;
-	ao::uvector<std::complex<float>> _modelBuffer;
+	aocommon::UVector<float> _weightBuffer, _imagingWeightBuffer;
+	aocommon::UVector<std::complex<float>> _modelBuffer;
 	std::unique_ptr<std::ofstream> _modelDataFile;
 	std::unique_ptr<std::fstream> _imagingWeightsFile;
 	int _fd;
-	PolarizationEnum _polarization;
+	aocommon::PolarizationEnum _polarization;
 	size_t _polarizationCountInFile;
 	
 	struct MetaHeader
@@ -153,8 +169,8 @@ private:
 	struct MetaRecord
 	{
 		double u, v, w, time;
-		uint16_t antenna1, antenna2, dataDescId;
-		static constexpr size_t BINARY_SIZE = 8*4 + 2*3;
+		uint16_t antenna1, antenna2, dataDescId, fieldId;
+		static constexpr size_t BINARY_SIZE = 8*4 + 2*4;
 		void read(std::istream& str)
 		{
 			str.read(reinterpret_cast<char*>(&u), sizeof(double));
@@ -164,6 +180,7 @@ private:
 			str.read(reinterpret_cast<char*>(&antenna1), sizeof(uint16_t));
 			str.read(reinterpret_cast<char*>(&antenna2), sizeof(uint16_t));
 			str.read(reinterpret_cast<char*>(&dataDescId), sizeof(uint16_t));
+			str.read(reinterpret_cast<char*>(&fieldId), sizeof(uint16_t));
 		}
 		void write(std::ostream& str) const
 		{
@@ -174,6 +191,7 @@ private:
 			str.write(reinterpret_cast<const char*>(&antenna1), sizeof(uint16_t));
 			str.write(reinterpret_cast<const char*>(&antenna2), sizeof(uint16_t));
 			str.write(reinterpret_cast<const char*>(&dataDescId), sizeof(uint16_t));
+			str.write(reinterpret_cast<const char*>(&fieldId), sizeof(uint16_t));
 		}
 	};
 	struct PartHeader
@@ -185,7 +203,7 @@ private:
 	} _partHeader;
 	
 	static std::string getFilenamePrefix(const std::string& msPath, const std::string& tempDir);
-	static std::string getPartPrefix(const std::string& msPath, size_t partIndex, PolarizationEnum pol, size_t dataDescId, const std::string& tempDir);
+	static std::string getPartPrefix(const std::string& msPath, size_t partIndex, aocommon::PolarizationEnum pol, size_t dataDescId, const std::string& tempDir);
 	static std::string getMetaFilename(const std::string& msPath, const std::string& tempDir, size_t dataDescId);
 };
 
