@@ -12,8 +12,7 @@
 
 #include "../units/fluxdensity.h"
 
-MultiScaleAlgorithm::MultiScaleAlgorithm(ImageBufferAllocator& allocator, FFTWManager& fftwManager, double beamSize, double pixelScaleX, double pixelScaleY) :
-	_allocator(allocator),
+MultiScaleAlgorithm::MultiScaleAlgorithm(FFTWManager& fftwManager, double beamSize, double pixelScaleX, double pixelScaleY) :
 	_fftwManager(fftwManager),
 	_width(0),
 	_height(0),
@@ -21,8 +20,8 @@ MultiScaleAlgorithm::MultiScaleAlgorithm(ImageBufferAllocator& allocator, FFTWMa
 	_beamSizeInPixels(beamSize / std::max(pixelScaleX, pixelScaleY)),
 	_multiscaleScaleBias(0.6),
 	_multiscaleGain(0.2),
-	_multiscaleNormalizeResponse(false),
 	_scaleShape(MultiScaleTransforms::TaperedQuadraticShape),
+	_maxScales(0),
 	_trackPerScaleMasks(false), _usePerScaleMasks(false),
 	_fastSubMinorLoop(true), _trackComponents(false)
 {
@@ -46,7 +45,7 @@ MultiScaleAlgorithm::~MultiScaleAlgorithm()
 }
 
 
-double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet& modelSet, const ao::uvector<const double*>& psfs, size_t width, size_t height, bool& reachedMajorThreshold)
+double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet& modelSet, const aocommon::UVector<const double*>& psfs, size_t width, size_t height, bool& reachedMajorThreshold)
 {
 	// Rough overview of the procedure:
 	// Convolve integrated image (all scales)
@@ -78,7 +77,7 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 	{
 		// Note that in a second round the nr of scales can be different (due to different width/height,
 		// e.g. caused by a different subdivision in parallel cleaning).
-		for(const ao::uvector<bool>& mask : _scaleMasks)
+		for(const aocommon::UVector<bool>& mask : _scaleMasks)
 		{
 			if(mask.size() != _width*_height)
 				throw std::runtime_error("Invalid automask size in multiscale algorithm");
@@ -91,7 +90,7 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 	if(_trackComponents)
 	{
 		if(_componentList == nullptr)
-			_componentList.reset(new ComponentList(_width, _height, _scaleInfos.size(), dirtySet.size(), _allocator));
+			_componentList.reset(new ComponentList(_width, _height, _scaleInfos.size(), dirtySet.size()));
 		else if(_componentList->Width() != _width || _componentList->Height() != _height)
 		{
 			throw std::runtime_error("Error in component list dimensions!");
@@ -103,17 +102,17 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 	bool hasHitThresholdInSubLoop = false;
 	size_t thresholdCountdown = std::max(size_t(8), _scaleInfos.size()*3/2);
 	
-	ImageBufferAllocator::Ptr scratch, scratchB, integratedScratch;
+	Image scratch, scratchB, integratedScratch;
 	// scratch and scratchB are used by the subminorloop, which convolves the images
 	// and requires therefore more space. This space depends on the scale, so here
 	// the required size for the largest scale is calculated.
 	size_t scratchWidth, scratchHeight;
 	getConvolutionDimensions(_scaleInfos.size()-1, scratchWidth, scratchHeight);
-	_allocator.Allocate(scratchWidth*scratchHeight, scratch);
-	_allocator.Allocate(scratchWidth*scratchHeight, scratchB);
-	_allocator.Allocate(_width*_height, integratedScratch);
-	std::unique_ptr<std::unique_ptr<ImageBufferAllocator::Ptr[]>[]> convolvedPSFs(
-		new std::unique_ptr<ImageBufferAllocator::Ptr[]>[dirtySet.PSFCount()]);
+	scratch = Image(scratchWidth, scratchHeight);
+	scratchB = Image(scratchWidth, scratchHeight);
+	integratedScratch = Image(_width, _height);
+	std::unique_ptr<std::unique_ptr<Image[]>[]> convolvedPSFs(
+		new std::unique_ptr<Image[]>[dirtySet.PSFCount()]);
 	dirtySet.GetIntegratedPSF(integratedScratch.data(), psfs);
 	convolvePSFs(convolvedPSFs[0], integratedScratch.data(), scratch.data(), true);
 
@@ -132,7 +131,7 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 	findActiveScaleConvolvedMaxima(dirtySet, integratedScratch.data(), scratch.data(), true, tools.get());
 	if(!selectMaximumScale(scaleWithPeak))
 	{
-		Logger::Warn << "No peak found during multi-scale cleaning! Aborting deconvolution.\n";
+		_logReceiver->Warn << "No peak found during multi-scale cleaning! Aborting deconvolution.\n";
 		reachedMajorThreshold = false;
 		return 0.0;
 	}
@@ -147,21 +146,21 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 		isFinalThreshold = true;
 	}
 	
-	Logger::Info << "Starting multi-scale cleaning. Start peak="
+	_logReceiver->Info << "Starting multi-scale cleaning. Start peak="
 		<< FluxDensity::ToNiceString(_scaleInfos[scaleWithPeak].maxUnnormalizedImageValue * _scaleInfos[scaleWithPeak].biasFactor)
 		<< ", major iteration threshold=" << FluxDensity::ToNiceString(firstThreshold);
 	if(isFinalThreshold)
-		Logger::Info << " (final)";
-	Logger::Info << '\n';
+		_logReceiver->Info << " (final)";
+	_logReceiver->Info << '\n';
 	
-	std::unique_ptr<ImageBufferAllocator::Ptr[]> doubleConvolvedPSFs(
-		new ImageBufferAllocator::Ptr[dirtySet.PSFCount()]);
+	std::unique_ptr<Image[]> doubleConvolvedPSFs(
+		new Image[dirtySet.PSFCount()]);
 	for(size_t i=0; i!=dirtySet.PSFCount(); ++i)
 	{
-		_allocator.Allocate(_width*_height, doubleConvolvedPSFs[i]);
+		doubleConvolvedPSFs[i] = Image(_width, _height);
 	}
 	
-	ImageSet individualConvolvedImages(&dirtySet.Table(), dirtySet.Allocator(), dirtySet.Settings(), _width, _height);
+	ImageSet individualConvolvedImages(&dirtySet.Table(), dirtySet.Settings(), _width, _height);
 	
 	//
 	// The minor iteration loop
@@ -173,7 +172,7 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 		thresholdCountdown > 0)
 	{
 		// Create double-convolved PSFs & individually convolved images for this scale
-		ao::uvector<double*> transformList;
+		aocommon::UVector<double*> transformList;
 		for(size_t i=0; i!=dirtySet.PSFCount(); ++i)
 		{
 			double* psf = getConvolvedPSF(i, scaleWithPeak, convolvedPSFs);
@@ -185,7 +184,7 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 			memcpy(individualConvolvedImages[i], dirtySet[i], _width*_height*sizeof(double));
 			transformList.push_back(individualConvolvedImages[i]);
 		}
-		if(scaleWithPeak != 0)
+		if(_scaleInfos[scaleWithPeak].scale != 0.0)
 		{
 			tools->MultiScaleTransform(&msTransforms, transformList, scratch.data(), _scaleInfos[scaleWithPeak].scale);
 			//msTransforms.Transform(transformList, scratch.data(), _scaleInfos[scaleWithPeak].scale);
@@ -202,11 +201,11 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 			firstSubIterationThreshold = firstThreshold;
 			if(!hasHitThresholdInSubLoop)
 			{
-				Logger::Info << "Subminor loop is near minor loop threshold. Initiating countdown.\n";
+				_logReceiver->Info << "Subminor loop is near minor loop threshold. Initiating countdown.\n";
 				hasHitThresholdInSubLoop = true;
 			}
 			thresholdCountdown--;
-			Logger::Info << '(' << thresholdCountdown << ") ";
+			_logReceiver->Info << '(' << thresholdCountdown << ") ";
 		}
 		// TODO we could chose to run the non-fast loop until we hit e.g. 10 iterations in a scale,
 		// because the fast loop takes more constant time and is only efficient when doing
@@ -217,51 +216,51 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 			size_t subMinorStartIteration = _iterationNumber;
 			size_t convolutionWidth, convolutionHeight;
 			getConvolutionDimensions(scaleWithPeak, convolutionWidth, convolutionHeight);
-			SubMinorLoop clarkLoop(_width, _height, convolutionWidth, convolutionHeight);
-			clarkLoop.SetIterationInfo(_iterationNumber, MaxNIter());
-			clarkLoop.SetThreshold(firstSubIterationThreshold / _scaleInfos[scaleWithPeak].biasFactor, subIterationGainThreshold / _scaleInfos[scaleWithPeak].biasFactor);
-			clarkLoop.SetGain(_scaleInfos[scaleWithPeak].gain);
-			clarkLoop.SetAllowNegativeComponents(AllowNegativeComponents());
-			clarkLoop.SetStopOnNegativeComponent(StopOnNegativeComponents());
+			SubMinorLoop subLoop(_width, _height, convolutionWidth, convolutionHeight, *_logReceiver);
+			subLoop.SetIterationInfo(_iterationNumber, MaxNIter());
+			subLoop.SetThreshold(firstSubIterationThreshold / _scaleInfos[scaleWithPeak].biasFactor, subIterationGainThreshold / _scaleInfos[scaleWithPeak].biasFactor);
+			subLoop.SetGain(_scaleInfos[scaleWithPeak].gain);
+			subLoop.SetAllowNegativeComponents(AllowNegativeComponents());
+			subLoop.SetStopOnNegativeComponent(StopOnNegativeComponents());
 			const size_t
 				scaleBorder = size_t(ceil(_scaleInfos[scaleWithPeak].scale*0.5)),
 				horBorderSize = std::max<size_t>(round(width * _cleanBorderRatio), scaleBorder),
 				vertBorderSize = std::max<size_t>(round(height * _cleanBorderRatio), scaleBorder);
-			clarkLoop.SetCleanBorders(horBorderSize, vertBorderSize);
+			subLoop.SetCleanBorders(horBorderSize, vertBorderSize);
 			if(!_rmsFactorImage.empty())
-				clarkLoop.SetRMSFactorImage(_rmsFactorImage);
+				subLoop.SetRMSFactorImage(_rmsFactorImage);
 			if(_usePerScaleMasks)
-				clarkLoop.SetMask(_scaleMasks[scaleWithPeak].data());
+				subLoop.SetMask(_scaleMasks[scaleWithPeak].data());
 			else if(_cleanMask)
-				clarkLoop.SetMask(_cleanMask);
-			clarkLoop.SetSpectralFitter(&Fitter());
+				subLoop.SetMask(_cleanMask);
+			subLoop.SetSpectralFitter(&Fitter());
 			
-			ao::uvector<const double*> clarkPSFs(dirtySet.PSFCount());
-			for(size_t psfIndex=0; psfIndex!=clarkPSFs.size(); ++psfIndex)
-				clarkPSFs[psfIndex] = doubleConvolvedPSFs[psfIndex].data();
+			aocommon::UVector<const double*> subPSFs(dirtySet.PSFCount());
+			for(size_t psfIndex=0; psfIndex!=subPSFs.size(); ++psfIndex)
+				subPSFs[psfIndex] = doubleConvolvedPSFs[psfIndex].data();
 			
-			clarkLoop.Run(individualConvolvedImages, clarkPSFs);
+			subLoop.Run(individualConvolvedImages, subPSFs);
 			
-			_iterationNumber = clarkLoop.CurrentIteration();
+			_iterationNumber = subLoop.CurrentIteration();
 			_scaleInfos[scaleWithPeak].nComponentsCleaned += (_iterationNumber - subMinorStartIteration);
-			_scaleInfos[scaleWithPeak].totalFluxCleaned += clarkLoop.FluxCleaned();
+			_scaleInfos[scaleWithPeak].totalFluxCleaned += subLoop.FluxCleaned();
 			
 			for(size_t imageIndex=0; imageIndex!=dirtySet.size(); ++imageIndex)
 			{
 				// TODO this can be multi-threaded if each thread has its own temporaries
 				double *psf = getConvolvedPSF(dirtySet.PSFIndex(imageIndex), scaleWithPeak, convolvedPSFs);
-				clarkLoop.CorrectResidualDirty(_fftwManager, scratch.data(), scratchB.data(), integratedScratch.data(), imageIndex, dirtySet[imageIndex],  psf);
+				subLoop.CorrectResidualDirty(_fftwManager, scratch.data(), scratchB.data(), integratedScratch.data(), imageIndex, dirtySet[imageIndex],  psf);
 				
-				clarkLoop.GetFullIndividualModel(imageIndex, scratch.data());
+				subLoop.GetFullIndividualModel(imageIndex, scratch.data());
 				if(imageIndex==0)
 				{
 					if(_trackPerScaleMasks)
-						clarkLoop.UpdateAutoMask(_scaleMasks[scaleWithPeak].data());
+						subLoop.UpdateAutoMask(_scaleMasks[scaleWithPeak].data());
 					if(_trackComponents)
-						clarkLoop.UpdateComponentList(*_componentList, scaleWithPeak);
+						subLoop.UpdateComponentList(*_componentList, scaleWithPeak);
 				}
 				if(_scaleInfos[scaleWithPeak].scale != 0.0)
-					tools->MultiScaleTransform(&msTransforms, ao::uvector<double*>{scratch.data()}, integratedScratch.data(), _scaleInfos[scaleWithPeak].scale);
+					tools->MultiScaleTransform(&msTransforms, aocommon::UVector<double*>{scratch.data()}, integratedScratch.data(), _scaleInfos[scaleWithPeak].scale);
 				double* model = modelSet[imageIndex];
 				for(size_t i=0; i!=_width*_height; ++i)
 					model[i] += scratch.data()[i];
@@ -275,7 +274,7 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 				std::fabs(maxScaleInfo.maxUnnormalizedImageValue * maxScaleInfo.biasFactor) > firstSubIterationThreshold &&
 				(!StopOnNegativeComponents() || _scaleInfos[scaleWithPeak].maxUnnormalizedImageValue>=0.0) )
 			{
-				ao::uvector<double> componentValues;
+				aocommon::UVector<double> componentValues;
 				measureComponentValues(componentValues, scaleWithPeak, individualConvolvedImages);
 				PerformSpectralFit(componentValues.data());
 				
@@ -307,7 +306,7 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 				// Find maximum for this scale
 				individualConvolvedImages.GetLinearIntegrated(integratedScratch.data());
 				findPeakDirect(integratedScratch.data(), scratch.data(), scaleWithPeak);
-				Logger::Debug << "Scale now " << std::fabs(_scaleInfos[scaleWithPeak].maxUnnormalizedImageValue * _scaleInfos[scaleWithPeak].biasFactor) << '\n';
+				_logReceiver->Debug << "Scale now " << std::fabs(_scaleInfos[scaleWithPeak].maxUnnormalizedImageValue * _scaleInfos[scaleWithPeak].biasFactor) << '\n';
 				
 				++_iterationNumber;
 			}
@@ -316,9 +315,15 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 		activateScales(scaleWithPeak);
 		
 		findActiveScaleConvolvedMaxima(dirtySet, integratedScratch.data(), scratch.data(), false, tools.get());
-		selectMaximumScale(scaleWithPeak);
 		
-		Logger::Info << "Iteration " << _iterationNumber << ", scale " << round(_scaleInfos[scaleWithPeak].scale) << " px : " << FluxDensity::ToNiceString(_scaleInfos[scaleWithPeak].maxUnnormalizedImageValue*_scaleInfos[scaleWithPeak].biasFactor) << " at " << _scaleInfos[scaleWithPeak].maxImageValueX << ',' << _scaleInfos[scaleWithPeak].maxImageValueY << '\n';
+		if(!selectMaximumScale(scaleWithPeak))
+		{
+			_logReceiver->Warn << "No peak found in main loop of multi-scale cleaning! Aborting deconvolution.\n";
+			reachedMajorThreshold = false;
+			return 0.0;
+		}
+		
+		_logReceiver->Info << "Iteration " << _iterationNumber << ", scale " << round(_scaleInfos[scaleWithPeak].scale) << " px : " << FluxDensity::ToNiceString(_scaleInfos[scaleWithPeak].maxUnnormalizedImageValue*_scaleInfos[scaleWithPeak].biasFactor) << " at " << _scaleInfos[scaleWithPeak].maxImageValueX << ',' << _scaleInfos[scaleWithPeak].maxImageValueY << '\n';
 	}
 	
 	bool
@@ -327,13 +332,13 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 		//finalThresholdReached = std::fabs(_scaleInfos[scaleWithPeak].maxUnnormalizedImageValue * _scaleInfos[scaleWithPeak].biasFactor) <= _threshold;
 	
 	if(maxIterReached)
-		Logger::Info << "Cleaning finished because maximum number of iterations was reached.\n";
+		_logReceiver->Info << "Cleaning finished because maximum number of iterations was reached.\n";
 	else if(negativeReached)
-		Logger::Info << "Cleaning finished because a negative component was found.\n";
+		_logReceiver->Info << "Cleaning finished because a negative component was found.\n";
 	else if(isFinalThreshold)
-		Logger::Info << "Cleaning finished because the final threshold was reached.\n";
+		_logReceiver->Info << "Cleaning finished because the final threshold was reached.\n";
 	else
-		Logger::Info << "Minor loop finished, continuing cleaning after inversion/prediction round.\n";
+		_logReceiver->Info << "Minor loop finished, continuing cleaning after inversion/prediction round.\n";
 	
 	reachedMajorThreshold = !maxIterReached && !isFinalThreshold && !negativeReached;
 	return _scaleInfos[scaleWithPeak].maxUnnormalizedImageValue * _scaleInfos[scaleWithPeak].biasFactor;
@@ -358,12 +363,12 @@ void MultiScaleAlgorithm::initializeScaleInfo()
 				
 				scale *= 2.0;
 				++scaleIndex;
-			} while(scale < std::min(_width, _height)*0.5);
+			} while(scale < std::min(_width, _height)*0.5 && (_maxScales == 0 || scaleIndex < _maxScales));
 		}
 		else {
 			while(!_scaleInfos.empty() && _scaleInfos.back().scale >= std::min(_width, _height) * 0.5)
 			{
-				Logger::Info << "Scale size " << _scaleInfos.back().scale << " does not fit in cleaning region: removing scale.\n";
+				_logReceiver->Info << "Scale size " << _scaleInfos.back().scale << " does not fit in cleaning region: removing scale.\n";
 				_scaleInfos.erase(_scaleInfos.begin() + _scaleInfos.size() - 1);
 			}
 		}
@@ -381,18 +386,18 @@ void MultiScaleAlgorithm::initializeScaleInfo()
 	}
 }
 
-void MultiScaleAlgorithm::convolvePSFs(std::unique_ptr<ImageBufferAllocator::Ptr[]>& convolvedPSFs, const double* psf, double* tmp, bool isIntegrated)
+void MultiScaleAlgorithm::convolvePSFs(std::unique_ptr<Image[]>& convolvedPSFs, const double* psf, double* tmp, bool isIntegrated)
 {
 	MultiScaleTransforms msTransforms(_fftwManager, _width, _height, _scaleShape);
-	convolvedPSFs.reset(new ImageBufferAllocator::Ptr[_scaleInfos.size()]);
+	convolvedPSFs.reset(new Image[_scaleInfos.size()]);
 	if(isIntegrated)
-		Logger::Info << "Scale info:\n";
+		_logReceiver->Info << "Scale info:\n";
 	const double firstAutoScaleSize = _beamSizeInPixels * 2.0;
 	for(size_t scaleIndex=0; scaleIndex!=_scaleInfos.size(); ++scaleIndex)
 	{
 		ScaleInfo& scaleEntry = _scaleInfos[scaleIndex];
 		
-		_allocator.Allocate(_width*_height, convolvedPSFs[scaleIndex]);
+		convolvedPSFs[scaleIndex] = Image(_width, _height);
 		memcpy(convolvedPSFs[scaleIndex].data(), psf, _width*_height*sizeof(double));
 		
 		if(isIntegrated)
@@ -406,24 +411,23 @@ void MultiScaleAlgorithm::convolvePSFs(std::unique_ptr<ImageBufferAllocator::Ptr
 			//scaleEntry.biasFactor = std::max(1.0,
 			//	scaleEntry.psfPeak * scaleInfos[0].kernelPeak /
 			//	(scaleEntry.kernelPeak * scaleInfos[0].psfPeak));
-			double responseNormalization = _multiscaleNormalizeResponse ? scaleEntry.psfPeak : 1.0;
 			double expTerm;
 			if(scaleEntry.scale == 0.0 || _scaleInfos.size() < 2)
 				expTerm = 0.0;
 			else
 				expTerm = log2(scaleEntry.scale / firstAutoScaleSize);
-			scaleEntry.biasFactor = pow(_multiscaleScaleBias, -double(expTerm)) * 1.0 / responseNormalization;
+			scaleEntry.biasFactor = pow(_multiscaleScaleBias, -double(expTerm)) * 1.0;
 			
 			// I tried this, but wasn't perfect:
 			// _gain * _scaleInfos[0].kernelPeak / scaleEntry.kernelPeak;
-			scaleEntry.gain = _gain * _scaleInfos[0].psfPeak / scaleEntry.psfPeak;
+			scaleEntry.gain = _gain / scaleEntry.psfPeak;
 			
 			scaleEntry.isActive = true;
 			
 			if(scaleEntry.scale == 0.0)
 				memcpy(convolvedPSFs[scaleIndex].data(), psf, _width*_height*sizeof(double));
 			
-			Logger::Info << "- Scale " << round(scaleEntry.scale) << ", bias factor=" << round(scaleEntry.biasFactor*10.0)/10.0 << ", psfpeak=" << scaleEntry.psfPeak << ", gain=" << scaleEntry.gain << ", kernel peak=" << scaleEntry.kernelPeak << '\n';
+			_logReceiver->Info << "- Scale " << round(scaleEntry.scale) << ", bias factor=" << round(scaleEntry.biasFactor*10.0)/10.0 << ", psfpeak=" << scaleEntry.psfPeak << ", gain=" << scaleEntry.gain << ", kernel peak=" << scaleEntry.kernelPeak << '\n';
 		}
 		else {
 			if(scaleEntry.scale != 0.0)
@@ -438,9 +442,9 @@ void MultiScaleAlgorithm::findActiveScaleConvolvedMaxima(const ImageSet& imageSe
 	//ImageBufferAllocator::Ptr convolvedImage;
 	//_allocator.Allocate(_width*_height, convolvedImage);
 	imageSet.GetLinearIntegrated(integratedScratch);
-	ao::uvector<double> transformScales;
-	ao::uvector<size_t> transformIndices;
-	std::vector<ao::uvector<bool>> transformScaleMasks;
+	aocommon::UVector<double> transformScales;
+	aocommon::UVector<size_t> transformIndices;
+	std::vector<aocommon::UVector<bool>> transformScaleMasks;
 	for(size_t scaleIndex=0; scaleIndex!=_scaleInfos.size(); ++scaleIndex)
 	{
 		ScaleInfo& scaleEntry = _scaleInfos[scaleIndex];
@@ -462,7 +466,7 @@ void MultiScaleAlgorithm::findActiveScaleConvolvedMaxima(const ImageSet& imageSe
 	}
 	std::vector<ThreadedDeconvolutionTools::PeakData> results;
 	
-	tools->FindMultiScalePeak(&msTransforms, &_allocator, integratedScratch, transformScales, results, _allowNegativeComponents, _cleanMask, transformScaleMasks, _cleanBorderRatio, _rmsFactorImage, reportRMS);
+	tools->FindMultiScalePeak(&msTransforms, integratedScratch, transformScales, results, _allowNegativeComponents, _cleanMask, transformScaleMasks, _cleanBorderRatio, _rmsFactorImage, reportRMS);
 	
 	for(size_t i=0; i!=results.size(); ++i)
 	{
@@ -476,20 +480,20 @@ void MultiScaleAlgorithm::findActiveScaleConvolvedMaxima(const ImageSet& imageSe
 	}
 	if(reportRMS)
 	{
-		Logger::Info << "RMS per scale: {";
+		_logReceiver->Info << "RMS per scale: {";
 		for(size_t scaleIndex=0; scaleIndex!=_scaleInfos.size(); ++scaleIndex)
 		{
 			ScaleInfo& scaleEntry = _scaleInfos[scaleIndex];
 			//double rmsBias = _scaleInfos[0].rms / scaleEntry.rms;
 			if(scaleIndex != 0)
-				Logger::Info << ", ";
-			Logger::Info << round(scaleEntry.scale) << ": " << FluxDensity::ToNiceString(scaleEntry.rms);
+				_logReceiver->Info << ", ";
+			_logReceiver->Info << round(scaleEntry.scale) << ": " << FluxDensity::ToNiceString(scaleEntry.rms);
 			// This can be made an option later:
 			// scaleEntry.biasFactor = rmsBias;
 			// However, at large scales the RMS is not a good estimator of the significance, because
 			// at large scales also the signal looks noise like, and increases thereby the RMS.
 		}
-		Logger::Info << "}\n";
+		_logReceiver->Info << "}\n";
 	}
 }
 
@@ -524,27 +528,27 @@ void MultiScaleAlgorithm::activateScales(size_t scaleWithLastPeak)
 		bool doActivate = i == scaleWithLastPeak || /*i == runnerUp ||*/ std::fabs(_scaleInfos[i].maxUnnormalizedImageValue) * _scaleInfos[i].biasFactor > std::fabs(_scaleInfos[scaleWithLastPeak].maxUnnormalizedImageValue) * (1.0-_gain) * _scaleInfos[scaleWithLastPeak].biasFactor;
 		if(!_scaleInfos[i].isActive && doActivate)
 		{
-			Logger::Debug << "Scale " << _scaleInfos[i].scale << " is now significant and is activated.\n";
+			_logReceiver->Debug << "Scale " << _scaleInfos[i].scale << " is now significant and is activated.\n";
 			_scaleInfos[i].isActive = true;
 		}
 		else if(_scaleInfos[i].isActive && !doActivate) {
-			Logger::Debug << "Scale " << _scaleInfos[i].scale << " is insignificant and is deactivated.\n";
+			_logReceiver->Debug << "Scale " << _scaleInfos[i].scale << " is insignificant and is deactivated.\n";
 			_scaleInfos[i].isActive = false;
 		}
 	}
 }
 
-void MultiScaleAlgorithm::measureComponentValues(ao::uvector<double>& componentValues, size_t scaleIndex, ImageSet& imageSet)
+void MultiScaleAlgorithm::measureComponentValues(aocommon::UVector<double>& componentValues, size_t scaleIndex, ImageSet& imageSet)
 {
 	const ScaleInfo& scale = _scaleInfos[scaleIndex];
 	componentValues.resize(imageSet.size());
-	Logger::Debug << "Measuring " << scale.maxImageValueX << ',' << scale.maxImageValueY << ", scale " << scale.scale << ", integrated=" << scale.maxUnnormalizedImageValue << ":";
+	_logReceiver->Debug << "Measuring " << scale.maxImageValueX << ',' << scale.maxImageValueY << ", scale " << scale.scale << ", integrated=" << scale.maxUnnormalizedImageValue << ":";
 	for(size_t i=0; i!=imageSet.size(); ++i)
 	{
 		componentValues[i] = imageSet[i][scale.maxImageValueX + scale.maxImageValueY*_width];
-		Logger::Debug << ' ' << componentValues[i];
+		_logReceiver->Debug << ' ' << componentValues[i];
 	}
-	Logger::Debug << '\n';
+	_logReceiver->Debug << '\n';
 }
 
 void MultiScaleAlgorithm::addComponentToModel(double* model, size_t scaleWithPeak, double componentValue)
@@ -566,7 +570,7 @@ void MultiScaleAlgorithm::addComponentToModel(double* model, size_t scaleWithPea
 	}
 }
 
-double* MultiScaleAlgorithm::getConvolvedPSF(size_t psfIndex, size_t scaleIndex, const std::unique_ptr<std::unique_ptr<ImageBufferAllocator::Ptr[]>[]>& convolvedPSFs)
+double* MultiScaleAlgorithm::getConvolvedPSF(size_t psfIndex, size_t scaleIndex, const std::unique_ptr<std::unique_ptr<Image[]>[]>& convolvedPSFs)
 {
 	return convolvedPSFs[psfIndex][scaleIndex].data();
 }
@@ -602,6 +606,19 @@ void MultiScaleAlgorithm::findPeakDirect(const double* image, double* scratch, s
 		scaleInfo.maxNormalizedImageValue = get_optional_value_or(maxValue, 0.0) / _rmsFactorImage[scaleInfo.maxImageValueX + scaleInfo.maxImageValueY * _width];
 }
 
+static size_t calculateGoodFFTSize(size_t n)
+{
+size_t bestfac=2*n;
+/* NOTE: Starting from f2=2 here instead from f2=1 as usual, because the
+		result needs to be even. */
+for (size_t f2=2; f2<bestfac; f2*=2)
+	for (size_t f23=f2; f23<bestfac; f23*=3)
+		for (size_t f235=f23; f235<bestfac; f235*=5)
+			for (size_t f2357=f235; f2357<bestfac; f2357*=7)
+				if (f2357>=n) bestfac=f2357;
+return bestfac;
+}
+
 void MultiScaleAlgorithm::getConvolutionDimensions(size_t scaleIndex, size_t& width, size_t& height) const
 {
 	double scale = _scaleInfos[scaleIndex].scale;
@@ -609,13 +626,11 @@ void MultiScaleAlgorithm::getConvolutionDimensions(size_t scaleIndex, size_t& wi
 	// It's supposed to be a balance between diverging runs caused by
 	// insufficient padding on one hand, and taking up too much memory on the other.
 	// I've seen divergence when padding=1.1, _width=1500, max scale=726 and conv width=1650.
-	// Diverged occurred on scale 363.
+	// Divergence occurred on scale 363.
 	// Was solved with conv width=2250. 2250 = 1.1*(363*factor + 1500)  --> factor = 1.5
 	// And solved with conv width=2000. 2000 = 1.1*(363*factor + 1500)  --> factor = 0.8
 	width = ceil(_convolutionPadding * (scale*1.5 + _width));
 	height = ceil(_convolutionPadding * (scale*1.5 + _height));
-	if(width%2 != 0)
-		++width;
-	if(height%2 != 0)
-		++height;
+	width = calculateGoodFFTSize(width);
+	height = calculateGoodFFTSize(height);
 }
