@@ -11,9 +11,11 @@ class ProgressBar
 	
 		void SetProgress(size_t taskIndex, size_t taskCount);
 		
+		ProgressBar& operator=(ProgressBar&& rhs);
+		
 	private:
-		const std::string _taskDescription;
-		unsigned _displayedDots;
+		std::string _taskDescription;
+		int _displayedDots;
 };
 
 #endif
