@@ -4,7 +4,7 @@
 #include "../fitswriter.h"
 #include "../fitsreader.h"
 
-#include "imagebufferallocator.h"
+#include "../wsclean/logger.h"
 
 #include <string.h>
 #include <set>
@@ -12,7 +12,7 @@
 class CachedImageSet
 {
 public:
-	CachedImageSet() : _polCount(0), _freqCount(0), _allocator(nullptr), _image()
+	CachedImageSet() : _polCount(0), _freqCount(0), _image()
 	{ }
 	
 	~CachedImageSet()
@@ -24,14 +24,13 @@ public:
 	CachedImageSet(const CachedImageSet& source) = delete;
 	CachedImageSet& operator=(const CachedImageSet& source) = delete;
 	
-	void Initialize(const FitsWriter& writer, size_t polCount, size_t freqCount, const std::string& prefix, ImageBufferAllocator& allocator)
+	void Initialize(const FitsWriter& writer, size_t polCount, size_t freqCount, const std::string& prefix)
 	{
 		_writer = writer;
 		_polCount = polCount;
 		_freqCount = freqCount;
 		_prefix = prefix;
 		_image.reset();
-		_allocator = &allocator;
 	}
 	
 	void SetFitsWriter(const FitsWriter& writer)
@@ -39,13 +38,13 @@ public:
 		_writer = writer;
 	}
 	
-	void Load(double* image, PolarizationEnum polarization, size_t freqIndex, bool isImaginary) const
+	void Load(double* image, aocommon::PolarizationEnum polarization, size_t freqIndex, bool isImaginary) const
 	{
 		if(_writer.Width() == 0 || _writer.Height() == 0)
 			throw std::runtime_error("Writer is not set.");
 		Logger::Debug << "Loading " << name(polarization, freqIndex, isImaginary) << '\n';
 		if(_polCount == 1 && _freqCount == 1)
-			if(_image == nullptr)
+			if(_image.empty())
 				throw std::runtime_error("Loading image before store");
 			else
 				std::copy(_image.data(), _image.data() + _writer.Width()*_writer.Height(), image);
@@ -55,16 +54,16 @@ public:
 		}
 	}
 	
-	void Store(const double* image, PolarizationEnum polarization, size_t freqIndex, bool isImaginary)
+	void Store(const double* image, aocommon::PolarizationEnum polarization, size_t freqIndex, bool isImaginary)
 	{
 		if(_writer.Width() == 0 || _writer.Height() == 0)
 			throw std::runtime_error("Writer is not set.");
 		Logger::Debug << "Storing " << name(polarization, freqIndex, isImaginary) << '\n';
 		if(_polCount == 1 && _freqCount == 1)
 		{
-			if(_image == nullptr)
+			if(_image.empty())
 			{
-				_allocator->Allocate(_writer.Width() * _writer.Height(), _image);
+				_image = Image(_writer.Width(), _writer.Height());
 			}
 			std::copy(image, image + _writer.Width() * _writer.Height(), _image.data());
 			//memcpy(_image, image, _writer.Width() * _writer.Height() * sizeof(double));
@@ -77,18 +76,18 @@ public:
 	}
 	
 private:
-	std::string name(PolarizationEnum polarization, size_t freqIndex, bool isImaginary) const
+	std::string name(aocommon::PolarizationEnum polarization, size_t freqIndex, bool isImaginary) const
 	{
 		if(_freqCount == 1)
 		{
 			if(isImaginary)
-				return _prefix + '-' + Polarization::TypeToShortString(polarization) + "i-tmp.fits";
+				return _prefix + '-' + aocommon::Polarization::TypeToShortString(polarization) + "i-tmp.fits";
 			else
-				return _prefix + '-' + Polarization::TypeToShortString(polarization) + "-tmp.fits";
+				return _prefix + '-' + aocommon::Polarization::TypeToShortString(polarization) + "-tmp.fits";
 		}
 		else {
 			std::ostringstream str;
-			str <<  _prefix + '-' + Polarization::TypeToShortString(polarization);
+			str <<  _prefix + '-' + aocommon::Polarization::TypeToShortString(polarization);
 			if(isImaginary)
 				str << 'i';
 			str << '-';
@@ -103,8 +102,7 @@ private:
 	size_t _polCount, _freqCount;
 	std::string _prefix;
 	
-	ImageBufferAllocator* _allocator;
-	ImageBufferAllocator::Ptr _image;
+	Image _image;
 	std::set<std::string> _storedNames;
 };
 
