@@ -9,11 +9,11 @@
 class CommandLine
 {
 public:
-	static bool Parse(class WSClean& wsclean, int argc, char *argv[]);
+	static bool Parse(class WSClean& wsclean, int argc, char *argv[], bool isSlave);
 	static void Run(class WSClean& wsclean);
 	
 private:
-	static void deprecated(const std::string& param, const std::string& replacement);
+	static void deprecated(bool isSlave, const std::string& param, const std::string& replacement);
 	static void printHeader();
 	static void printHelp();
 	static size_t parse_size_t(const char* param, const char* name);
