@@ -38,15 +38,15 @@ void ImageOperations::FitBeamSize(const WSCleanSettings& settings, double& bMaj,
 	bMin = bMin*0.5*(settings.pixelScaleX + settings.pixelScaleY);
 }
 
-void ImageOperations::DetermineBeamSize(const WSCleanSettings& settings, double& bMaj, double& bMin, double& bPA, const double* image, double theoreticBeam)
+void ImageOperations::DetermineBeamSize(const WSCleanSettings& settings, double& bMaj, double& bMin, double& bPA, double& bTheoretical, const double* image, double initialEstimate)
 {
-	double theoreticBeamWithTaper = theoreticBeam;
+	bTheoretical = initialEstimate;
 	if(settings.gaussianTaperBeamSize != 0.0)
 	{
-		if(settings.gaussianTaperBeamSize > theoreticBeamWithTaper)
+		if(settings.gaussianTaperBeamSize > bTheoretical)
 		{
-			theoreticBeamWithTaper = settings.gaussianTaperBeamSize;
-			Logger::Debug << "Beam is tapered; using " << Angle::ToNiceString(theoreticBeamWithTaper) << " as initial value in PSF fitting.\n";
+			bTheoretical = settings.gaussianTaperBeamSize;
+			Logger::Debug << "Beam is tapered; using " << Angle::ToNiceString(bTheoretical) << " as initial value in PSF fitting.\n";
 		}
 	}
 	if(settings.manualBeamMajorSize != 0.0)
@@ -56,16 +56,16 @@ void ImageOperations::DetermineBeamSize(const WSCleanSettings& settings, double&
 		bPA = settings.manualBeamPA;
 	} else if(settings.fittedBeam)
 	{
-		FitBeamSize(settings, bMaj, bMin, bPA, image, theoreticBeamWithTaper*2.0/(settings.pixelScaleX + settings.pixelScaleY));
+		FitBeamSize(settings, bMaj, bMin, bPA, image, bTheoretical*2.0/(settings.pixelScaleX + settings.pixelScaleY));
 		Logger::Info << "major=" << Angle::ToNiceString(bMaj) << ", minor=" <<
 		Angle::ToNiceString(bMin) << ", PA=" << Angle::ToNiceString(bPA) << ", theoretical=" <<
-		Angle::ToNiceString(theoreticBeamWithTaper)<< ".\n";
+		Angle::ToNiceString(bTheoretical)<< ".\n";
 	}
 	else if(settings.theoreticBeam) {
-		bMaj = theoreticBeamWithTaper;
-		bMin = theoreticBeamWithTaper;
+		bMaj = bTheoretical;
+		bMin = bTheoretical;
 		bPA = 0.0;
-		Logger::Info << "Beam size is " << Angle::ToNiceString(theoreticBeamWithTaper) << '\n';
+		Logger::Info << "Beam size is " << Angle::ToNiceString(bTheoretical) << '\n';
 	} else {
 		bMaj = std::numeric_limits<double>::quiet_NaN();
 		bMin = std::numeric_limits<double>::quiet_NaN();
@@ -73,11 +73,11 @@ void ImageOperations::DetermineBeamSize(const WSCleanSettings& settings, double&
 	}
 }
 
-void ImageOperations::MakeMFSImage(const WSCleanSettings& settings, const std::vector<OutputChannelInfo>& infoPerChannel, OutputChannelInfo& mfsInfo, const string& suffix, size_t intervalIndex, PolarizationEnum pol, bool isImaginary, bool isPSF)
+void ImageOperations::MakeMFSImage(const WSCleanSettings& settings, const std::vector<OutputChannelInfo>& infoPerChannel, OutputChannelInfo& mfsInfo, const string& suffix, size_t intervalIndex, aocommon::PolarizationEnum pol, bool isImaginary, bool isPSF)
 {
 	double lowestFreq = 0.0, highestFreq = 0.0;
 	const size_t size = settings.trimmedImageWidth * settings.trimmedImageHeight;
-	ao::uvector<double> mfsImage(size, 0.0), addedImage(size), weightImage(size, 0.0);
+	aocommon::UVector<double> mfsImage(size, 0.0), addedImage(size), weightImage(size, 0.0);
 	double weightSum = 0.0;
 	FitsWriter writer;
 	for(size_t ch=0; ch!=settings.channelsOut; ++ch)
@@ -128,8 +128,11 @@ void ImageOperations::MakeMFSImage(const WSCleanSettings& settings, const std::v
 				break;
 			}
 		}
+		double pixelScale = std::min(settings.pixelScaleX, settings.pixelScaleY);
+		if(smallestTheoreticBeamSize < pixelScale)
+			smallestTheoreticBeamSize = pixelScale;
 		
-		ImageOperations::DetermineBeamSize(settings, mfsInfo.beamMaj, mfsInfo.beamMin, mfsInfo.beamPA, mfsImage.data(), smallestTheoreticBeamSize);
+		ImageOperations::DetermineBeamSize(settings, mfsInfo.beamMaj, mfsInfo.beamMin, mfsInfo.beamPA, mfsInfo.theoreticBeamSize, mfsImage.data(), smallestTheoreticBeamSize);
 	}
 	if(std::isfinite(mfsInfo.beamMaj))
 		writer.SetBeamInfo(mfsInfo.beamMaj, mfsInfo.beamMin, mfsInfo.beamPA);
@@ -145,7 +148,7 @@ void ImageOperations::MakeMFSImage(const WSCleanSettings& settings, const std::v
 	writer.Write(mfsName, mfsImage.data());
 }
 
-void ImageOperations::RenderMFSImage(const WSCleanSettings& settings, const OutputChannelInfo& mfsInfo, size_t intervalIndex, PolarizationEnum pol, bool isImaginary, bool isPBCorrected)
+void ImageOperations::RenderMFSImage(const WSCleanSettings& settings, const OutputChannelInfo& mfsInfo, size_t intervalIndex, aocommon::PolarizationEnum pol, bool isImaginary, bool isPBCorrected)
 {
 	const size_t size = settings.trimmedImageWidth * settings.trimmedImageHeight;
 	
@@ -153,7 +156,7 @@ void ImageOperations::RenderMFSImage(const WSCleanSettings& settings, const Outp
 	std::string postfix = isPBCorrected ? "-pb.fits" : ".fits";
 	FitsReader residualReader(mfsPrefix + "-residual" + postfix);
 	FitsReader modelReader(mfsPrefix + "-model" + postfix);
-	ao::uvector<double> image(size), modelImage(size);
+	aocommon::UVector<double> image(size), modelImage(size);
 	residualReader.Read(image.data());
 	modelReader.Read(modelImage.data());
 	
