@@ -15,7 +15,9 @@
 #include <casacore/measures/Measures/MPosition.h>
 #include <casacore/measures/Measures/MCPosition.h>
 #include <casacore/measures/TableMeasures/ScalarMeasColumn.h>
+
 #include <casacore/tables/Tables/ArrColDesc.h>
+#include <casacore/tables/Tables/TableRecord.h>
 
 MSGridderBase::MSData::MSData() : msIndex(0), matchingRows(0), totalRowsProcessed(0)
 { }
@@ -45,6 +47,35 @@ MSGridderBase::MSGridderBase() :
 MSGridderBase::~MSGridderBase()
 { }
 
+int64_t MSGridderBase::getAvailableMemory(double memFraction, double absMemLimit)
+{
+	static bool isFirst = true;
+	long int pageCount = sysconf(_SC_PHYS_PAGES), pageSize = sysconf(_SC_PAGE_SIZE);
+	int64_t memory = (int64_t) pageCount * (int64_t) pageSize;
+	double memSizeInGB = (double) memory / (1024.0*1024.0*1024.0);
+	if(memFraction == 1.0 && absMemLimit == 0.0 && isFirst) {
+		Logger::Info << "Detected " << round(memSizeInGB*10.0)/10.0 << " GB of system memory, usage not limited.\n";
+	}
+	else {
+		double limitInGB = memSizeInGB*memFraction;
+		if(absMemLimit!=0.0 && limitInGB > absMemLimit)
+			limitInGB = absMemLimit;
+		if(isFirst) {
+			Logger::Info << "Detected " << round(memSizeInGB*10.0)/10.0 << " GB of system memory, usage limited to " << round(limitInGB*10.0)/10.0 << " GB (frac=" << round(memFraction*1000.0)/10.0 << "%, ";
+			if(absMemLimit == 0.0)
+				Logger::Info << "no limit)\n";
+			else
+				Logger::Info << "limit=" << round(absMemLimit*10.0)/10.0 << "GB)\n";
+		}
+		
+		memory = int64_t((double) pageCount * (double) pageSize * memFraction);
+		if(absMemLimit!=0.0 && double(memory) > double(1024.0*1024.0*1024.0) * absMemLimit)
+			memory = int64_t(double(absMemLimit) * double(1024.0*1024.0*1024.0));
+	}
+	isFirst = false;
+	return memory;
+}
+
 void MSGridderBase::GetPhaseCentreInfo(casacore::MeasurementSet& ms, size_t fieldId, double& ra, double& dec, double& dl, double& dm)
 {
 	casacore::MSAntenna aTable = ms.antenna();
@@ -53,10 +84,11 @@ void MSGridderBase::GetPhaseCentreInfo(casacore::MeasurementSet& ms, size_t fiel
 	casacore::MPosition::ScalarColumn antPosColumn(aTable, aTable.columnName(casacore::MSAntennaEnums::POSITION));
 	casacore::MPosition ant1Pos = antPosColumn(0);
 	
+	size_t fieldRow = (fieldId == MSSelection::ALL_FIELDS) ? 0 : fieldId;
 	casacore::MEpoch::ScalarColumn timeColumn(ms, ms.columnName(casacore::MSMainEnums::TIME));
 	casacore::MSField fTable(ms.field());
 	casacore::MDirection::ScalarColumn phaseDirColumn(fTable, fTable.columnName(casacore::MSFieldEnums::PHASE_DIR));
-	casacore::MDirection phaseDir = phaseDirColumn(fieldId);
+	casacore::MDirection phaseDir = phaseDirColumn(fieldRow);
 	casacore::MEpoch curtime = timeColumn(0);
 	casacore::MeasFrame frame(ant1Pos, curtime);
 	casacore::MDirection::Ref j2000Ref(casacore::MDirection::J2000, frame);
@@ -117,15 +149,26 @@ void MSGridderBase::calculateWLimits(MSGridderBase::MSData& msData)
 	MultiBandData selectedBand = msData.SelectedBand();
 	std::vector<float> weightArray(selectedBand.MaxChannels() * NPolInMSProvider);
 	msData.msProvider->Reset();
+	double curTimestep = -1, firstTime = -1, lastTime = -1;
+	size_t nTimesteps = 0;
 	while(msData.msProvider->CurrentRowAvailable())
 	{
-		size_t dataDescId;
-		double uInM, vInM, wInM;
-		msData.msProvider->ReadMeta(uInM, vInM, wInM, dataDescId);
-		const BandData& curBand = selectedBand[dataDescId];
-		double wHi = fabs(wInM / curBand.SmallestWavelength());
-		double wLo = fabs(wInM / curBand.LongestWavelength());
-		double baselineInM = sqrt(uInM*uInM + vInM*vInM + wInM*wInM);
+		MSProvider::MetaData metaData;
+		msData.msProvider->ReadMeta(metaData);
+		
+		if(curTimestep != metaData.time)
+		{
+			curTimestep = metaData.time;
+			++nTimesteps;
+			if(firstTime == -1)
+				firstTime = curTimestep;
+			lastTime = curTimestep;
+		}
+		
+		const BandData& curBand = selectedBand[metaData.dataDescId];
+		double wHi = fabs(metaData.wInM / curBand.SmallestWavelength());
+		double wLo = fabs(metaData.wInM / curBand.LongestWavelength());
+		double baselineInM = sqrt(metaData.uInM*metaData.uInM + metaData.vInM*metaData.vInM + metaData.wInM*metaData.wInM);
 		double halfWidth = 0.5*ImageWidth(), halfHeight = 0.5*ImageHeight();
 		if(wHi > msData.maxW || wLo < msData.minW || baselineInM / curBand.SmallestWavelength() > msData.maxBaselineUVW)
 		{
@@ -134,22 +177,22 @@ void MSGridderBase::calculateWLimits(MSGridderBase::MSData& msData)
 			for(size_t ch=0; ch!=curBand.ChannelCount(); ++ch)
 			{
 				const double wavelength = curBand.ChannelWavelength(ch);
-				double wInL = wInM/wavelength;
+				double wInL = metaData.wInM/wavelength;
 				msData.maxWWithFlags = std::max(msData.maxWWithFlags, fabs(wInL));
 				if(*weightPtr != 0.0)
 				{
 					double
-						uInL = uInM/wavelength, vInL = vInM/wavelength,
+						uInL = metaData.uInM/wavelength, vInL = metaData.vInM/wavelength,
 						x = uInL * PixelSizeX() * ImageWidth(),
 						y = vInL * PixelSizeY() * ImageHeight(),
-						imagingWeight = PrecalculatedWeightInfo()->GetWeight(uInL, vInL);
+						imagingWeight = GetImageWeights()->GetWeight(uInL, vInL);
 					if(imagingWeight != 0.0)
 					{
-						if(floor(x) > -halfWidth  && ceil(x) < halfWidth &&
-							floor(y) > -halfHeight && ceil(y) < halfHeight)
+						if(std::floor(x) > -halfWidth  && std::ceil(x) < halfWidth &&
+							std::floor(y) > -halfHeight && std::ceil(y) < halfHeight)
 						{
-							msData.maxW = std::max(msData.maxW, fabs(wInL));
-							msData.minW = std::min(msData.minW, fabs(wInL));
+							msData.maxW = std::max(msData.maxW, std::fabs(wInL));
+							msData.minW = std::min(msData.minW, std::fabs(wInL));
 							msData.maxBaselineUVW = std::max(msData.maxBaselineUVW, baselineInM / wavelength);
 							msData.maxBaselineInM = std::max(msData.maxBaselineInM, baselineInM);
 						}
@@ -174,6 +217,11 @@ void MSGridderBase::calculateWLimits(MSGridderBase::MSData& msData)
 	{
 		Logger::Debug << "Discarded data has higher w value of " << msData.maxWWithFlags << " lambda.\n";
 	}
+	
+	if(lastTime == firstTime || nTimesteps < 2)
+		msData.integrationTime = 1;
+	else
+		msData.integrationTime = (lastTime - firstTime) / (nTimesteps - 1);
 }
 
 template void MSGridderBase::calculateWLimits<1>(MSGridderBase::MSData& msData);
@@ -211,7 +259,8 @@ void MSGridderBase::initializeMetaData(casacore::MeasurementSet& ms, size_t fiel
 	
 	casacore::MSField fTable = ms.field();
 	casacore::ScalarColumn<casacore::String> fieldNameColumn(fTable, fTable.columnName(casacore::MSField::NAME));
-	_fieldName = fieldNameColumn(fieldId);
+	size_t fieldRow = (fieldId == MSSelection::ALL_FIELDS) ? 0 : fieldId;
+	_fieldName = fieldNameColumn(fieldRow);
 }
 
 void MSGridderBase::initializeMeasurementSet(MSGridderBase::MSData& msData, MetaDataCache::Entry& cacheEntry, bool isCacheInitialized)
@@ -225,9 +274,9 @@ void MSGridderBase::initializeMeasurementSet(MSGridderBase::MSData& msData, Meta
 	
 	calculateMSLimits(msData.SelectedBand(), msProvider.StartTime());
 	
-	initializePhaseCentre(*ms, Selection(msData.msIndex).FieldId());
+	initializePhaseCentre(*ms, Selection(msData.msIndex).FieldIds()[0]);
 	
-	initializeMetaData(*ms, Selection(msData.msIndex).FieldId());
+	initializeMetaData(*ms, Selection(msData.msIndex).FieldIds()[0]);
 	
 	if(isCacheInitialized)
 	{
@@ -236,9 +285,10 @@ void MSGridderBase::initializeMeasurementSet(MSGridderBase::MSData& msData, Meta
 		msData.minW = cacheEntry.minW;
 		msData.maxBaselineUVW = cacheEntry.maxBaselineUVW;
 		msData.maxBaselineInM = cacheEntry.maxBaselineInM;
+		msData.integrationTime = cacheEntry.integrationTime;
 	}
 	else {
-		if (msProvider.Polarization() == Polarization::Instrumental)
+		if (msProvider.Polarization() == aocommon::Polarization::Instrumental)
 			calculateWLimits<4>(msData);
 		else
 			calculateWLimits<1>(msData);
@@ -247,6 +297,7 @@ void MSGridderBase::initializeMeasurementSet(MSGridderBase::MSData& msData, Meta
 		cacheEntry.minW = msData.minW;
 		cacheEntry.maxBaselineUVW = msData.maxBaselineUVW;
 		cacheEntry.maxBaselineInM = msData.maxBaselineInM;
+		cacheEntry.integrationTime = msData.integrationTime;
 	}
 }
 
@@ -389,7 +440,7 @@ void MSGridderBase::readAndWeightVisibilities(MSProvider& msProvider, InversionR
 		double
 			u = rowData.uvw[0] / curBand.ChannelWavelength(ch),
 			v = rowData.uvw[1] / curBand.ChannelWavelength(ch),
-			imageWeight = PrecalculatedWeightInfo()->GetWeight(u, v);
+			imageWeight = GetImageWeights()->GetWeight(u, v);
 		_scratchWeights[ch] = imageWeight;
 		
 		for(size_t p=0; p!=PolarizationCount; ++p)
