@@ -4,7 +4,9 @@
 #include "measurementsetgridder.h"
 
 #include "../multibanddata.h"
-#include "../uvector.h"
+#include "../scheduling/metadatacache.h"
+
+#include <aocommon/uvector.h>
 
 #include <mutex>
 
@@ -68,24 +70,12 @@ public:
 	
 	const std::string& FieldName() const { return _fieldName; }
 	
-	class AverageBeamBase
-	{
-	public:
-			virtual ~AverageBeamBase() {}
-	};
-
-	struct MetaDataCache
-	{
-		struct Entry {
-			double minW, maxW, maxWWithFlags, maxBaselineUVW, maxBaselineInM;
-		};
-		std::vector<Entry> msDataVector;
-		std::unique_ptr<AverageBeamBase> averageBeam;
-	};
-	
-	void SetMetaDataCache(MetaDataCache* cache) { _metaDataCache = cache; }
+	void SetMetaDataCache(std::unique_ptr<MetaDataCache> cache) { _metaDataCache = std::move(cache); }
+	std::unique_ptr<MetaDataCache> AcquireMetaDataCache() { return std::move(_metaDataCache); }
 	
 protected:
+	int64_t getAvailableMemory(double memFraction, double absMemLimit);
+	
 	struct MSData
 	{
 		public:
@@ -98,6 +88,7 @@ protected:
 			size_t matchingRows, totalRowsProcessed;
 			double minW, maxW, maxWWithFlags, maxBaselineUVW, maxBaselineInM;
 			size_t rowStart, rowEnd;
+			double integrationTime;
 		
 			MultiBandData SelectedBand() const { return MultiBandData(bandData, startChannel, endChannel); }
 		private:
@@ -184,7 +175,7 @@ protected:
 	
 	void initializeMSDataVector(std::vector<MSData>& msDataVector);
 
-	struct MetaDataCache* _metaDataCache;
+	std::unique_ptr<struct MetaDataCache> _metaDataCache;
 
 	
 private:
@@ -211,7 +202,7 @@ private:
 	double _maxGriddedWeight;
 	double _visibilityWeightSum;
 	
-	ao::uvector<float> _scratchWeights;
+	aocommon::UVector<float> _scratchWeights;
 };
 
 #endif
