@@ -1,6 +1,8 @@
 #ifndef OBSERVATION_INFO_H
 #define OBSERVATION_INFO_H
 
+#include <string>
+
 struct ObservationInfo
 {
 	double phaseCentreRA = 0.0, phaseCentreDec = 0.0;
@@ -10,6 +12,9 @@ struct ObservationInfo
 	std::string telescopeName;
 	std::string observer;
 	std::string fieldName;
+	
+	void Serialize(class SerialOStream& stream) const;
+	void Unserialize(class SerialIStream& stream);
 };
 
 #endif
