@@ -1,6 +1,5 @@
 #include "wsmsgridder.h"
 
-#include "imagebufferallocator.h"
 #include "logger.h"
 
 #include "../imageweights.h"
@@ -12,40 +11,26 @@
 
 #include <casacore/ms/MeasurementSets/MeasurementSet.h>
 
-#include <iostream>
+#include <fftw3.h>
+
 #include <stdexcept>
 
-WSMSGridder::WSMSGridder(ImageBufferAllocator* imageAllocator, size_t threadCount, double memFraction, double absMemLimit) :
+WSMSGridder::WSMSGridder(size_t threadCount, double memFraction, double absMemLimit) :
 	MSGridderBase(),
 	_cpuCount(threadCount),
-	_laneBufferSize(std::max<size_t>(_cpuCount*2,1024)),
-	_imageBufferAllocator(imageAllocator)
+	_laneBufferSize(std::max<size_t>(_cpuCount*2,1024))
 {
-	long int pageCount = sysconf(_SC_PHYS_PAGES), pageSize = sysconf(_SC_PAGE_SIZE);
-	_memSize = (int64_t) pageCount * (int64_t) pageSize;
-	double memSizeInGB = (double) _memSize / (1024.0*1024.0*1024.0);
-	if(memFraction == 1.0 && absMemLimit == 0.0) {
-		Logger::Info << "Detected " << round(memSizeInGB*10.0)/10.0 << " GB of system memory, usage not limited.\n";
-	}
-	else {
-		double limitInGB = memSizeInGB*memFraction;
-		if(absMemLimit!=0.0 && limitInGB > absMemLimit)
-			limitInGB = absMemLimit;
-		Logger::Info << "Detected " << round(memSizeInGB*10.0)/10.0 << " GB of system memory, usage limited to " << round(limitInGB*10.0)/10.0 << " GB (frac=" << round(memFraction*1000.0)/10.0 << "%, ";
-		if(absMemLimit == 0.0)
-			Logger::Info << "no limit)\n";
-		else
-			Logger::Info << "limit=" << round(absMemLimit*10.0)/10.0 << "GB)\n";
-		
-		_memSize = int64_t((double) pageCount * (double) pageSize * memFraction);
-		if(absMemLimit!=0.0 && double(_memSize) > double(1024.0*1024.0*1024.0) * absMemLimit)
-			_memSize = int64_t(double(absMemLimit) * double(1024.0*1024.0*1024.0));
-	}
+	_memSize = getAvailableMemory(memFraction, absMemLimit);
+	
+	// We do this once here. WStackingGridder does this too, but by default only for the float
+	// variant of fftw. FFTResampler does double fft's multithreaded, hence this needs to be done
+	// here too.
+	fftw_make_planner_thread_safe();
 }
-		
+
 void WSMSGridder::countSamplesPerLayer(MSData& msData)
 {
-	ao::uvector<size_t> sampleCount(ActualWGridSize(), 0);
+	aocommon::UVector<size_t> sampleCount(ActualWGridSize(), 0);
 	size_t total = 0;
 	msData.matchingRows = 0;
 	msData.msProvider->Reset();
@@ -69,9 +54,9 @@ void WSMSGridder::countSamplesPerLayer(MSData& msData)
 		msData.msProvider->NextRow();
 	}
 	Logger::Debug << "Visibility count per layer: ";
-	for(ao::uvector<size_t>::const_iterator i=sampleCount.begin(); i!=sampleCount.end(); ++i)
+	for(size_t& count : sampleCount)
 	{
-		Logger::Debug << *i << ' ';
+		Logger::Debug << count << ' ';
 	}
 	Logger::Debug << "\nTotal nr. of visibilities to be gridded: " << total << '\n';
 }
@@ -101,7 +86,7 @@ size_t WSMSGridder::getSuggestedWGridSize() const
 	{
 		// When nwlayers is lower than the nr of cores, we cannot parallellize well. 
 		// However, we don't want extra w-layers if we are low on mem, as that might slow down the process
-		double memoryRequired = double(_cpuCount) * double(sizeof(double))*double(_actualInversionWidth*_actualInversionHeight);
+		double memoryRequired = double(_cpuCount) * double(sizeof(GridderType::num_t))*double(_actualInversionWidth*_actualInversionHeight);
 		if(4.0 * memoryRequired < double(_memSize))
 		{
 			Logger::Info <<
@@ -125,9 +110,9 @@ void WSMSGridder::gridMeasurementSet(MSData &msData)
 {
 	const MultiBandData selectedBand(msData.SelectedBand());
 	_gridder->PrepareBand(selectedBand);
-	ao::uvector<std::complex<float>> modelBuffer(selectedBand.MaxChannels());
-	ao::uvector<float> weightBuffer(selectedBand.MaxChannels());
-	ao::uvector<bool> isSelected(selectedBand.MaxChannels());
+	aocommon::UVector<std::complex<float>> modelBuffer(selectedBand.MaxChannels());
+	aocommon::UVector<float> weightBuffer(selectedBand.MaxChannels());
+	aocommon::UVector<bool> isSelected(selectedBand.MaxChannels());
 	
 	// Samples of the same w-layer are collected in a buffer
 	// before they are written into the lane. This is done because writing
@@ -143,7 +128,7 @@ void WSMSGridder::gridMeasurementSet(MSData &msData)
 	}
 	
 	InversionRow newItem;
-	ao::uvector<std::complex<float>> newItemData(selectedBand.MaxChannels());
+	aocommon::UVector<std::complex<float>> newItemData(selectedBand.MaxChannels());
 	newItem.data = newItemData.data();
 			
 	size_t rowsRead = 0;
@@ -221,7 +206,7 @@ void WSMSGridder::finishInversionWorkThreads()
 	_inversionCPULanes.clear();
 }
 
-void WSMSGridder::workThreadPerSample(ao::lane<InversionWorkSample>* workLane)
+void WSMSGridder::workThreadPerSample(aocommon::Lane<InversionWorkSample>* workLane)
 {
 	size_t bufferSize = std::max<size_t>(8u, workLane->capacity()/8);
 	bufferSize = std::min<size_t>(128,std::min(bufferSize, workLane->capacity()));
@@ -241,7 +226,7 @@ void WSMSGridder::predictMeasurementSet(MSData &msData)
 	
 	size_t rowsProcessed = 0;
 	
-	ao::lane<PredictionWorkItem>
+	aocommon::Lane<PredictionWorkItem>
 		calcLane(_laneBufferSize+_cpuCount),
 		writeLane(_laneBufferSize);
 	set_lane_debug_name(calcLane, "Prediction calculation lane (buffered) containing full row data");
@@ -302,7 +287,7 @@ void WSMSGridder::predictMeasurementSet(MSData &msData)
 	writeThread.join();
 }
 
-void WSMSGridder::predictCalcThread(ao::lane<PredictionWorkItem>* inputLane, ao::lane<PredictionWorkItem>* outputLane)
+void WSMSGridder::predictCalcThread(aocommon::Lane<PredictionWorkItem>* inputLane, aocommon::Lane<PredictionWorkItem>* outputLane)
 {
 	lane_write_buffer<PredictionWorkItem> writeBuffer(outputLane, _laneBufferSize);
 	
@@ -315,7 +300,7 @@ void WSMSGridder::predictCalcThread(ao::lane<PredictionWorkItem>* inputLane, ao:
 	}
 }
 
-void WSMSGridder::predictWriteThread(ao::lane<PredictionWorkItem>* predictionWorkLane, const MSData* msData)
+void WSMSGridder::predictWriteThread(aocommon::Lane<PredictionWorkItem>* predictionWorkLane, const MSData* msData)
 {
 	lane_read_buffer<PredictionWorkItem> buffer(predictionWorkLane, std::min(_laneBufferSize, predictionWorkLane->capacity()));
 	PredictionWorkItem workItem;
@@ -330,13 +315,13 @@ void WSMSGridder::Invert()
 	std::vector<MSData> msDataVector;
 	initializeMSDataVector(msDataVector);
 	
-	_gridder.reset(new WStackingGridder(_actualInversionWidth, _actualInversionHeight, _actualPixelSizeX, _actualPixelSizeY, _cpuCount, _imageBufferAllocator, AntialiasingKernelSize(), OverSamplingFactor()));
+	_gridder.reset(new GridderType(_actualInversionWidth, _actualInversionHeight, _actualPixelSizeX, _actualPixelSizeY, _cpuCount, AntialiasingKernelSize(), OverSamplingFactor()));
 	_gridder->SetGridMode(GridMode());
 	if(HasDenormalPhaseCentre())
 		_gridder->SetDenormalPhaseCentre(PhaseCentreDL(), PhaseCentreDM());
 	_gridder->SetIsComplex(IsComplex());
-	//_imager->SetImageConjugatePart(Polarization() == Polarization::YX && IsComplex());
-	_gridder->PrepareWLayers(ActualWGridSize(), double(_memSize)*(7.0/10.0), _minW, _maxW);
+	//_imager->SetImageConjugatePart(Polarization() == aocommon::Polarization::YX && IsComplex());
+	_gridder->PrepareWLayers(ActualWGridSize(), double(_memSize)*(6.0/10.0), _minW, _maxW);
 	
 	if(Verbose() && Logger::IsVerbose())
 	{
@@ -391,6 +376,12 @@ void WSMSGridder::Invert()
 		Logger::Info << ", effective count after weighting: " << EffectiveGriddedVisibilityCount();
 	Logger::Info << '\n';
 	
+	_realImage = _gridder->RealImageDouble();
+	if(IsComplex())
+		_imaginaryImage = _gridder->ImaginaryImageDouble();
+	else
+		_imaginaryImage = Image();
+	
 	if(ImageWidth()!=_actualInversionWidth || ImageHeight()!=_actualInversionHeight)
 	{
 		// Interpolate the image
@@ -399,23 +390,19 @@ void WSMSGridder::Invert()
 		
 		if(IsComplex())
 		{
-			ImageBufferAllocator::Ptr resizedReal = _imageBufferAllocator->AllocatePtr(ImageWidth() * ImageHeight());
-			ImageBufferAllocator::Ptr resizedImag = _imageBufferAllocator->AllocatePtr(ImageWidth() * ImageHeight());
+			Image resizedReal(ImageWidth(), ImageHeight());
+			Image resizedImag(ImageWidth(), ImageHeight());
 			resampler.Start();
-			ImageBufferAllocator::Ptr
-				real = _gridder->RealImage(),
-				imaginary = _gridder->ImaginaryImage();
-			resampler.AddTask(real.data(), resizedReal.data());
-			resampler.AddTask(imaginary.data(), resizedImag.data());
+			resampler.AddTask(_realImage.data(), resizedReal.data());
+			resampler.AddTask(_imaginaryImage.data(), resizedImag.data());
 			resampler.Finish();
-			_gridder->ReplaceRealImageBuffer(std::move(resizedReal));
-			_gridder->ReplaceImaginaryImageBuffer(std::move(resizedImag));
+			_realImage = std::move(resizedReal);
+			_imaginaryImage = std::move(resizedImag);
 		}
 		else {
-			ImageBufferAllocator::Ptr resized = _imageBufferAllocator->AllocatePtr(ImageWidth() * ImageHeight());
-			ImageBufferAllocator::Ptr real = _gridder->RealImage();
-			resampler.Resample(real.data(), resized.data());
-			_gridder->ReplaceRealImageBuffer(std::move(resized));
+			Image resized = Image(ImageWidth(), ImageHeight());
+			resampler.Resample(_realImage.data(), resized.data());
+			_realImage = std::move(resized);
 		}
 	}
 	
@@ -424,40 +411,37 @@ void WSMSGridder::Invert()
 		Logger::Debug << "Trimming " << ImageWidth() << " x " << ImageHeight() << " -> " << TrimWidth() << " x " << TrimHeight() << '\n';
 		// Perform trimming
 		
-		ImageBufferAllocator::Ptr
-			trimmed = _imageBufferAllocator->AllocatePtr(TrimWidth() * TrimHeight()),
-			real = _gridder->RealImage();
-		Image::Trim(trimmed.data(), TrimWidth(), TrimHeight(), real.data(), ImageWidth(), ImageHeight());
-		_gridder->ReplaceRealImageBuffer(std::move(trimmed));
+		Image trimmed(TrimWidth(), TrimHeight());
+		Image::Trim(trimmed.data(), TrimWidth(), TrimHeight(), _realImage.data(), ImageWidth(), ImageHeight());
+		_realImage = std::move(trimmed);
 		
 		if(IsComplex())
 		{
-			ImageBufferAllocator::Ptr
-				trimmedImag = _imageBufferAllocator->AllocatePtr(TrimWidth() * TrimHeight()),
-				imag = _gridder->ImaginaryImage();
-			Image::Trim(trimmedImag.data(), TrimWidth(), TrimHeight(), imag.data(), ImageWidth(), ImageHeight());
-			_gridder->ReplaceImaginaryImageBuffer(std::move(trimmedImag));
+			Image trimmedImag = Image(TrimWidth(), TrimHeight());
+			Image::Trim(trimmedImag.data(), TrimWidth(), TrimHeight(), _imaginaryImage.data(), ImageWidth(), ImageHeight());
+			_imaginaryImage = std::move(trimmedImag);
 		}
 	}
+	Logger::Debug << "Inversion finished.\n";
 }
 
-void WSMSGridder::Predict(ImageBufferAllocator::Ptr real, ImageBufferAllocator::Ptr imaginary)
+void WSMSGridder::Predict(Image real, Image imaginary)
 {
-	if(imaginary==nullptr && IsComplex())
+	if(imaginary.empty() && IsComplex())
 		throw std::runtime_error("Missing imaginary in complex prediction");
-	if(imaginary!=0 && !IsComplex())
+	if(!imaginary.empty() && !IsComplex())
 		throw std::runtime_error("Imaginary specified in non-complex prediction");
 	
 	std::vector<MSData> msDataVector;
 	initializeMSDataVector(msDataVector);
 	
-	_gridder = std::unique_ptr<WStackingGridder>(new WStackingGridder(_actualInversionWidth, _actualInversionHeight, _actualPixelSizeX, _actualPixelSizeY, _cpuCount, _imageBufferAllocator, AntialiasingKernelSize(), OverSamplingFactor()));
+	_gridder = std::unique_ptr<GridderType>(new GridderType(_actualInversionWidth, _actualInversionHeight, _actualPixelSizeX, _actualPixelSizeY, _cpuCount, AntialiasingKernelSize(), OverSamplingFactor()));
 	_gridder->SetGridMode(GridMode());
 	if(HasDenormalPhaseCentre())
 		_gridder->SetDenormalPhaseCentre(PhaseCentreDL(), PhaseCentreDM());
 	_gridder->SetIsComplex(IsComplex());
-	//_imager->SetImageConjugatePart(Polarization() == Polarization::YX && IsComplex());
-	_gridder->PrepareWLayers(ActualWGridSize(), double(_memSize)*(7.0/10.0), _minW, _maxW);
+	//_imager->SetImageConjugatePart(Polarization() == aocommon::Polarization::YX && IsComplex());
+	_gridder->PrepareWLayers(ActualWGridSize(), double(_memSize)*(6.0/10.0), _minW, _maxW);
 	
 	if(Verbose())
 	{
@@ -465,39 +449,39 @@ void WSMSGridder::Predict(ImageBufferAllocator::Ptr real, ImageBufferAllocator::
 			countSamplesPerLayer(msDataVector[i]);
 	}
 	
-	ImageBufferAllocator::Ptr untrimmedReal, untrimmedImag;
+	Image untrimmedReal, untrimmedImag;
 	if(TrimWidth() != ImageWidth() || TrimHeight() != ImageHeight())
 	{
 		Logger::Debug << "Untrimming " << TrimWidth() << " x " << TrimHeight() << " -> " << ImageWidth() << " x " << ImageHeight() << '\n';
 		// Undo trimming (i.e., extend with zeros)
 		// The input is of size TrimWidth() x TrimHeight()
 		// This will make the model image of size ImageWidth() x ImageHeight()
-		_imageBufferAllocator->Allocate(ImageWidth() * ImageHeight(), untrimmedReal);
+		untrimmedReal = Image(ImageWidth(), ImageHeight());
 		Image::Untrim(untrimmedReal.data(), ImageWidth(), ImageHeight(), real.data(), TrimWidth(), TrimHeight());
 		real = std::move(untrimmedReal);
 		
 		if(IsComplex())
 		{
-			_imageBufferAllocator->Allocate(ImageWidth() * ImageHeight(), untrimmedImag);
+			untrimmedImag = Image(ImageWidth(), ImageHeight());
 			Image::Untrim(untrimmedImag.data(), ImageWidth(), ImageHeight(), imaginary.data(), TrimWidth(), TrimHeight());
 			imaginary = std::move(untrimmedImag);
 		}
 	}
 	
-	ImageBufferAllocator::Ptr resampledReal, resampledImag;
+	Image resampledReal, resampledImag;
 	if(ImageWidth()!=_actualInversionWidth || ImageHeight()!=_actualInversionHeight)
 	{
 		// Decimate the image
 		// Input is ImageWidth() x ImageHeight()
 		FFTResampler resampler(ImageWidth(), ImageHeight(), _actualInversionWidth, _actualInversionHeight, _cpuCount);
 		
-		_imageBufferAllocator->Allocate(ImageWidth() * ImageHeight(), resampledReal);
-		if(imaginary == nullptr)
+		resampledReal = Image(ImageWidth(), ImageHeight());
+		if(imaginary.empty())
 		{
 			resampler.Resample(real.data(), resampledReal.data());
 		}
 		else {
-			_imageBufferAllocator->Allocate(ImageWidth() * ImageHeight(), resampledImag);
+			resampledImag = Image(ImageWidth(), ImageHeight());
 			resampler.Start();
 			resampler.AddTask(real.data(), resampledReal.data());
 			resampler.AddTask(imaginary.data(), resampledImag.data());
@@ -507,7 +491,7 @@ void WSMSGridder::Predict(ImageBufferAllocator::Ptr real, ImageBufferAllocator::
 		real = std::move(resampledReal);
 	}
 	
-	if(imaginary == nullptr)
+	if(imaginary.empty())
 		_gridder->InitializePrediction(std::move(real));
 	else
 		_gridder->InitializePrediction(std::move(real), std::move(imaginary));
