@@ -4,15 +4,16 @@
 #include "msgridderbase.h"
 #include "wstackinggridder.h"
 
-#include "../lane.h"
 #include "../multibanddata.h"
-
-#include <complex>
-#include <memory>
+#include "../image.h"
 
 #include <casacore/casa/Arrays/Array.h>
 #include <casacore/tables/Tables/ArrayColumn.h>
 
+#include <aocommon/lane.h>
+
+#include <complex>
+#include <memory>
 #include <thread>
 
 namespace casacore {
@@ -23,22 +24,22 @@ class ImageBufferAllocator;
 class WSMSGridder : public MSGridderBase
 {
 	public:
-		WSMSGridder(class ImageBufferAllocator* imageAllocator, size_t threadCount, double memFraction, double absMemLimit);
+		typedef WStackingGridderF GridderType;
+		
+		WSMSGridder(size_t threadCount, double memFraction, double absMemLimit);
 	
 		virtual void Invert() final override;
 		
-		virtual void Predict(ImageBufferAllocator::Ptr image) final override { Predict(std::move(image), nullptr); }
-		virtual void Predict(ImageBufferAllocator::Ptr real, ImageBufferAllocator::Ptr imaginary) final override;
+		virtual void Predict(Image image) final override { Predict(std::move(image), Image()); }
+		virtual void Predict(Image real, Image imaginary) final override;
 		
-		virtual ImageBufferAllocator::Ptr ImageRealResult() final override { return _gridder->RealImage(); }
-		virtual ImageBufferAllocator::Ptr ImageImaginaryResult() final override {
+		virtual Image ImageRealResult() final override
+		{ return std::move(_realImage); }
+		virtual Image ImageImaginaryResult() final override {
 			if(!IsComplex())
 				throw std::runtime_error("No imaginary result available for non-complex inversion");
-			return _gridder->ImaginaryImage();
+			return std::move(_imaginaryImage);
 		}
-		virtual bool HasGriddingCorrectionImage() const final override { return GridMode() != NearestNeighbourGridding; }
-		virtual void GetGriddingCorrectionImage(double *image) const final override { _gridder->GetGriddingCorrectionImage(image); }
-		
 		virtual size_t ActualInversionWidth() const final override { return _actualInversionWidth; }
 		virtual size_t ActualInversionHeight() const final override { return _actualInversionHeight; }
 		
@@ -66,7 +67,7 @@ class WSMSGridder : public MSGridderBase
 
 		void predictMeasurementSet(MSData& msData);
 
-		void workThread(ao::lane<InversionRow>* workLane)
+		void workThread(aocommon::Lane<InversionRow>* workLane)
 		{
 			InversionRow workItem;
 			while(workLane->read(workItem))
@@ -78,17 +79,17 @@ class WSMSGridder : public MSGridderBase
 		
 		void startInversionWorkThreads(size_t maxChannelCount);
 		void finishInversionWorkThreads();
-		void workThreadPerSample(ao::lane<InversionWorkSample>* workLane);
+		void workThreadPerSample(aocommon::Lane<InversionWorkSample>* workLane);
 		
-		void predictCalcThread(ao::lane<PredictionWorkItem>* inputLane, ao::lane<PredictionWorkItem>* outputLane);
-		void predictWriteThread(ao::lane<PredictionWorkItem>* samplingWorkLane, const MSData* msData);
+		void predictCalcThread(aocommon::Lane<PredictionWorkItem>* inputLane, aocommon::Lane<PredictionWorkItem>* outputLane);
+		void predictWriteThread(aocommon::Lane<PredictionWorkItem>* samplingWorkLane, const MSData* msData);
 
-		std::unique_ptr<WStackingGridder> _gridder;
-		std::vector<ao::lane<InversionWorkSample>> _inversionCPULanes;
+		std::unique_ptr<GridderType> _gridder;
+		std::vector<aocommon::Lane<InversionWorkSample>> _inversionCPULanes;
 		std::vector<std::thread> _threadGroup;
 		size_t _cpuCount, _laneBufferSize;
 		int64_t _memSize;
-		ImageBufferAllocator* _imageBufferAllocator;
+		Image _realImage, _imaginaryImage;
 };
 
 #endif
